from rest_framework import permissions,status,views,viewsets
from rest_framework.response import Response
from rest_framework import generics

from django.core.mail import send_mail
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth import get_user_model
User = get_user_model()

from django.contrib.sites.shortcuts import get_current_site
from django.db import transaction
from rest_framework_jwt.settings import api_settings



from django.utils import timezone
from datetime import timedelta

from .permissions import IsAccountOwner
from .serializers import UserSerializer
from .models import UserActivation, ResetPasswordRequest
from .settings import AUTHENTICATION

from agape.signals import trigger


class UserViewSet(viewsets.ModelViewSet):
    """ Viewset that provides CRUD operations for user accounts
    
    Extends:
        viewsets.ModelViewSet

    """
    queryset = User.objects.all()
    serializer_class = UserSerializer
    context = 'user'

    
    def get_permissions(self):
        """ Set permission restrictions """

        # allow super admin to access all methods
        if self.request.user.is_superuser:
            return(permissions.AllowAny(),)
        
        # allow not logged in users to create accounts
        if self.request.method == 'POST':
            return(permissions.AllowAny(),)
        
        # required user to be logged in and account owner to view and edit
        return(permissions.IsAuthenticated(), IsAccountOwner() )
    

    def create(self,request,*args,**kwargs):    
        """ Perform account registration
            
        Accepts an email address and password as post data and creates a new user account.
        Sends an email to the user to activate the account before permitting login.
        """

         # create the user
        trigger(self.context+'.create:request',request,*args,**kwargs)

        data = request.data.copy()
        trigger(self.context+'.create:before',data)


        # check for existing account with given email
        try:
            duplicate = User.objects.get(email=request.data.get('email'))
            if duplicate:
                
                response = Response({
                    'status': 'Fail',
                    'message': 'This email is already in use.'
                }, status=status.HTTP_409_CONFLICT)

                trigger(self.context+'.create:fail',response)
                return response

                return 
        except:
            pass

        # set default user status
        data['status'] = 0 if AUTHENTICATION['REQUIRE_ACTIVATION'] else 1
        serializer = self.serializer_class(data=data)

        # validate supplied data
        if serializer.is_valid():
            
            # create the user account
            instance =  serializer.save()
            trigger(self.context+'.create:success',instance)

            # if account requires activation
            if AUTHENTICATION['REQUIRE_ACTIVATION']:

                # create and send the activation link
                self.dispatch_activation_email(instance,request)

            # remove password data from response
            serializer = self.serializer_class(instance)
            serialized_data = serializer.data
            trigger(self.context+'.create:serialize',serialized_data,instance)


            response = Response(serialized_data, status=status.HTTP_201_CREATED)
            trigger(self.context+'.create:response',response)
            return response
        
        else:
            # data not valid, return bad request response
            response = Response({
                'status': 'Fail',
                'message': 'Account could not be created with received data.'
            }, status=status.HTTP_400_BAD_REQUEST)
            trigger(self.context+'.create:fail',response)
            return response



    def partial_update(self, request, pk=None):
        """ Update user data. Ensure password is encrypted. """

        # confirm current password to commit credential changes
        if request.data.get('confirm_password') and not request.data.get('confirm_password') == "":
            user = User.objects.get(pk=pk)
            
            # authenticate the password
            auth = authenticate(email=user.email, password=request.data.get('confirm_password'))

        # throw error if current password not given
        else:
            return Response({
                'status': 'fail',
                'message': 'Current password incorrect.'
            }, status=status.HTTP_401_UNAUTHORIZED)


        # if a password was supplied
        if request.data.get('password') and not request.data.get('password') == "":

            if auth:
                # encrypt the password
                user.set_password( request.data.get('password') )
                # remove the password from the request data 
                request.data.pop('password');
            else:
                return Response({
                    'status': 'fail',
                    'message': 'Current password required.'
                }, status=status.HTTP_401_UNAUTHORIZED)     


        # perform update
        serializer = self.get_serializer(user,data=request.data,partial=True)
        if serializer.is_valid():
            serializer.save();
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(request.data, status=status.HTTP_400_BAD_REQUEST)


    def dispatch_activation_email(self, user, request):
        """ Send the activation email to the user when the database record is created.

        Creates a new "UserActivation" record in the database which contains a unique
        key. An email is then sent to the user with a link containing the key. Following
        this link will set the user account to active.

        Args:
            user (object): The user to send the activation link
        """

        # create the activation record
        activation = UserActivation.objects.create(user_id=user.id)

        # create the link
        current_site = get_current_site(request)
        protocol = 'https' if request.is_secure() else 'http' 
        link = '{}://{}/activate/{}'.format(protocol, current_site.domain, activation.key)

        try:
            # dispatch the email
            send_mail(
                'Activate your account at {}'.format(current_site.name),
                'Please activate your account by following this link \n\n{}\n\n-'.format(link),
                'test@example.com',
                [user.email],
                fail_silently=False
            )
        except Error as error:
            print( error )


class AuthenticationView(views.APIView):
    """ Handles authentication (logging in and out)
    """

    def post(self, request, format=None):
        """ Sign in to a user account .

        Returns a response with an authentication token
        """
        data = request.data
        email = data.get('email', None)
        password = data.get('password', None)


        # TODO: Find the user based on the supplied credentials
        # User can login with username, email, or phone number
        #
        # Once the user is found, authenticate the user

        # authenticate the user
        user = authenticate(email=email, password=password)

        # user authenticated successfully
        if user is not None:

            # if user is active
            if user.is_active:

                # login
                login(request, user)
                user.last_login = timezone.now()
                user.save()

                # create an authentication token
                jwt_payload_handler = api_settings.JWT_PAYLOAD_HANDLER
                jwt_encode_handler = api_settings.JWT_ENCODE_HANDLER
                
                payload = jwt_payload_handler(user)
                token = jwt_encode_handler(payload)

                # create the response
                response = {
                    'user': UserSerializer(user).data,
                    'token': token 
                    }
                return Response( response, status=status.HTTP_200_OK );

            # if user is not active
            else:
                return Response({
                    'status': 'unauthorized',
                    'message': 'This user has been disabled.'
                }, status=status.HTTP_401_UNAUTHORIZED)

        # could not authenticate user
        else:
            return Response({
                'status': 'unauthorized',
                'message': 'Invalid credentials.'
            }, status=status.HTTP_401_UNAUTHORIZED)    


    def delete(self, request, *args, **kwargs):
        """ Destroy a users authentication token, effectively signing out.
        """

        # If a user is signed in
        if request.user.is_authenticated():
            # sign out
            logout(request)
            return Response({}, status=status.HTTP_204_NO_CONTENT) 
        
        # otherwise respond with bad request
        else:
            # Do something for anonymous users.
            return Response({
                'status': 'Bad request',
                'message': 'There is no authenticated user to sign out.'
            }, status=status.HTTP_400_BAD_REQUEST)  


    def patch(self,request,*args,**kwargs):
        """ Activate a user account
        """

        key = request.data.get('key')
        if key == None:
            return Response({
                    'status': 'fail',
                    'message': 'No activation key given.',
                },  status=status.HTTP_400_BAD_REQUEST);

        # perform key lookup
        results = UserActivation.objects.filter(key=key).all()

        # no activation key found in database
        if len(results) == 0:
            return Response({
                    'status': 'fail',
                    'message': 'Could not activate account',
                },  status=status.HTTP_400_BAD_REQUEST);

        # user has been banned
        elif results[0] and results[0].user.status < 0:
            return Response({
                    'status': 'fail',
                    'message': 'This account has been disabled.',
                },  status=status.HTTP_400_BAD_REQUEST);              

        # found activation key, but user is already activated
        elif results[0] and results[0].activated:
            return Response({
                    'status': 'fail',
                    'message': 'This account has already been activated.',
                },  status=status.HTTP_409_CONFLICT);   


        # found activation key, activate account
        else:
            activation = results[0]
            activation.activated = timezone.now()
            activation.save()

            activation.user.status = 1;
            activation.user.save();
            return Response({
                    'status': 'succes',
                    'message': 'This account is now active.',
                },  status=status.HTTP_200_OK); 



    

class ResetPasswordRequestView(views.APIView):
    """ Send a password reset link to a user via email
    
    Will send a password eset link if no other request has been issued in the 
    past X minutes. Password reset links are only valid for the number of minutes
    specified in the configuration file. Defaults to 30.

    Extends:
        views.APIView    
    """
    @transaction.atomic
    def post(self,request,*args, **kwargs):
        """ Create a new reset password request. 

        Generates a new unique key which can be used to reset a users password. Sends
        an email containing a password reset link to the specified user.
        """

        # get key from uri
        email = request.data.get('email')

        # find user with given email
        user = User.objects.filter(email=email).all()

        # user does not exist, return 404
        if len(user) == 0:
            return Response({
                    'status': 'fail',
                    'message': 'No user with this email address.',
                },  status=status.HTTP_404_NOT_FOUND);

        # user does exist
        else:
            user = user[0]

        # if user has been disabled, throw an error
        if user.status == User.DISABLED:
            return Response({
                    'status': 'fail',
                    'message': 'This user has been disabled.',
                },  status=status.HTTP_400_BAD_REQUEST);
        
        # check if there is another password reset request sent in the past 30 minutes
        # TODO: Allow minutes to be configured via settings module
        time_threshold = timezone.now() - timedelta(minutes=AUTHENTICATION.get('RESET_PASSWORD_EXPIRES'))
        results = ResetPasswordRequest.objects.filter(user=user,status=ResetPasswordRequest.ENABLED, created__gt=time_threshold).order_by('-id')
                
        # if active request, use the same request link
        if len(results) > 0:
            instance = results[0]

        # if no active request, create a new request link
        else:
            instance = ResetPasswordRequest.objects.create(user=user)

        # create password reset instance
        self.dispatch_email(instance, request)

        return Response({
                'status': 'succes',
                'message': 'A link to reset your password has been sent to this email address.',
            },  status=status.HTTP_200_OK); 




    def get(self,request,*args, **kwargs):
        """ Check a reset password request for valididy and expiration
        """

        # get key from uri
        key = self.kwargs.get('key')

        # check key for validity
        response = self.validate_key(key)

        # if received Response, invalid key
        if isinstance(response, Response):
            # return the response
            return response
        # otherwise, confirm password reset link is valid
        else:
            return Response({
                    'status': 'success',
                    'message': 'Password reset key is valid.'
                }, status=status.HTTP_200_OK);


    @transaction.atomic
    def patch(self,request,*args, **kwargs):
        """Peform the password reset.
        
        Decorators:
            transaction.atomic
        
        Request Data:
            password (string): Email address of registered user

        Status:
            200: Success
            400: No password provided
            401: Link is no longer valid or has been used
            404: Reset code not found   
        """

        # get key from uri
        key = self.kwargs.get('key')

        # check key for validity
        response = self.validate_key(key)

        # if received Response, invalid key
        if isinstance(response, Response):
            # return the response
            return response
        # otherwise, we have a ResetPasswordRequest
        else:
            instance = response

        # return error if user did not supply a password
        if request.data.get('password', None) == None or request.data.get('password') == "":
            return Response({
                    'status': 'fail',
                    'message': 'No password set.'
                }, status=status.HTTP_400_BAD_REQUEST);
 

        # set the password
        user = instance.user
        user.set_password(request.data.get('password'))
        user.save()

        instance.used = timezone.now()
        instance.status = ResetPasswordRequest.USED
        instance.save()


        return Response({
                'status': 'success',
                'message': 'Password has been changed.'
            }, status=status.HTTP_200_OK);


    def validate_key(self,key):
        """ Check a key for validity.

        Checks that key exists, has not been disabled, and is not expired.

        Returns:
            Response object for invalid keys
            ResetPasswordRequest for valid keys
        """

        # get user from password reset items
        try:
            instance = ResetPasswordRequest.objects.get(key=key)
        except:
            # return 404 if no matching reset request
            return Response({
                    'status': 'fail',
                    'message': 'This password reset key is not valid.'
                }, status=status.HTTP_404_NOT_FOUND);

        # return error if link has been used already or is disabled
        if instance.status == ResetPasswordRequest.USED:
            return Response({
                    'status': 'fail',
                    'message': 'This password reset key has expired.'
                }, status=status.HTTP_410_GONE);

        elif instance.status == ResetPasswordRequest.DISABLED:
            return Response({
                    'status': 'used',
                    'message': 'This password reset key has been disabled.'
                }, status=status.HTTP_403_FORBIDDEN);

        # return error if link has expired
        elif instance.created < timezone.now() - timedelta(minutes=AUTHENTICATION.get('RESET_PASSWORD_EXPIRES')):
            return Response({
                    'status': 'fail',
                    'message': 'This password reset key has expired.'
                }, status=status.HTTP_410_GONE);

        # check that user is not disabled
        elif instance.user.status == User.DISABLED:
            return Response({
                    'status': 'fail',
                    'message': 'This account has been disabled.',
                },  status=status.HTTP_403_FORBIDDEN);

        else:
            return instance

    def dispatch_email(self, instance, request):
        """ Send an email to the user containing a link to reset their password.

        Args:
            instance (object): The ResetPasswordRequest instance
            request (object): The request object
        """

        # create the link
        current_site = get_current_site(request)
        protocol = 'https' if request.is_secure() else 'http' 
        link = '{}://{}/reset-password/{}'.format(protocol, current_site.domain, instance.key)

        # dispatch the email
        send_mail(
            'Password Reset',
            'Reset your password by following this link \n\n{}\n\n-'.format(link),
            'test@example.com',
            [instance.user.email],
            fail_silently=False
            )

class ValidateEmailView(views.APIView):
    """ Determine if an email is already registered with a user account."""


    def get(self,request):

        email = request.GET.get('email')
        userid = request.GET.get('userid')

        # check for existing account with given email
        duplicates = User.objects.filter(email=email)

        # exclude the account already registered with this email
        if userid:
            duplicates = duplicates.exclude(id=userid)

        results = duplicates.all()
        if len(results) > 0:
            return Response({
                'valid': False,
                'message': 'This email is already in use.'
            }, status=status.HTTP_200_OK)
        else:
            return Response({
                'valid': True
            }, status=status.HTTP_200_OK)



from django.contrib import admin

# Register your models here.
from django.contrib import admin
from .models import Contact

admin.site.register(Contact)
# admin.site.register(UserActivation)
# admin.site.register(ResetPasswordRequest)


from django.test import TestCase
from rest_framework.test import APIRequestFactory
from rest_framework.test import APIClient


import json
from django.utils import timezone
from datetime import timedelta
import datetime



class AgapeTests(object):

    def stringifyDatesAndTimes(self, data):
        for record in data:
            # for each item in create data
            for key, value in record.items():
                # if it is a datetime object
                if isinstance(value, datetime.datetime):
                    # format as a datetime string
                    record[key] = value.strftime('%Y-%m-%dT%H:%M:%S')
                 # if it is a date object
                elif isinstance(value,datetime.date):
                     # format as a date string
                    record[key] = value.strftime('%Y-%m-%d')




class ModelTests(AgapeTests):
     

    def test_create(self):

        i = 0
        while i < len(self.create_data):
            instance = self.model(**self.create_data[i])
            instance.save()

            # compare instance to expected data
            for key in self.expect_data[i].keys():
                instance_value = getattr(instance, key)
                expected_value = self.expect_data[i][key]
                self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )
            i+=1


    def test_retrieve(self):
        i = 0
        while i < len(self.create_data):
            instance = self.model(**self.create_data[i])
            instance.save()
            instance = None

            instance = self.model.objects.get(id=i+1)
            self.assertTrue(instance)
        
            # compare instance to expected data
            for key in self.expect_data[i].keys():
                instance_value = getattr(instance, key)
                expected_value = self.expect_data[i][key]
                self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )
            i+=1

    def test_update(self):

        i=0
        while i < len(self.create_data):
            instance = self.model(**self.create_data[i])
            instance.save()

            # update the instance
            for key, value in self.update_data[i].items():
                instance_value = setattr(instance, key, value )

            # save the instance
            instance.save()

            # retrieve the instance
            instance_id = instance.id
            instance = None
            instance = self.model.objects.get(id=instance_id)
        

        
            # compare instance to expected data
            for key in self.update_expect_data[i].keys():
                instance_value = getattr(instance, key)
                expected_value = self.update_expect_data[i][key]
                self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

            i+=1

    def test_delete(self):

        i=0
        while i < len(self.create_data):
            instance = self.model(**self.create_data[i])
            instance.save()
            
            # retrieve the instance
            instance_id = instance.id
            instance = None
            instance = self.model.objects.get(id=instance_id)
            self.assertTrue(instance)

            # delete the instance
            self.model.objects.filter(id=instance_id).delete()
        

            instance = self.model.objects.filter(id=instance_id).all()
            self.assertTrue(not instance)


            i+=1


    def test_list(self):

        i=0
        while i < len(self.update_data):
            instance = self.model(**self.create_data[i])
            instance.save()
            i+=1

        list = self.model.objects.all()
        self.assertEqual(len(list),2)


class SerializerTests(AgapeTests):


    def test_serialize(self):

        i=0
        while i < len(self.create_data):
            instance = self.model(**self.create_data[i])
            instance.save()

            serializer = self.serializer_class(instance)

            # compare instance to expected data
            for key in self.serialized_expect_data[i].keys():
                instance_value = serializer.data.get(key)
                expected_value = self.serialized_expect_data[i][key]
                self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )
            i+=1

    def test_create(self):

        i=0
        while i < len(self.create_data):

            # serializer data is valid
            serializer = self.serializer_class(data=self.serialized_create_data[i])
            self.assertTrue(serializer.is_valid(), 'Serializer data is valid')

            # create an instance via the serializer
            instance = serializer.create(serializer.validated_data)
            self.assertTrue(instance, 'Created instance from serializer data')

            i+=1


    def test_update(self):

        i=0
        while i < len(self.create_data):

            instance =self.model(**self.create_data[i])
            instance.save()

            serializer = self.serializer_class(instance,data=self.serialized_update_data[i],partial=True)
            self.assertTrue(serializer.is_valid(raise_exception=True), 'Serializer data is valid')

            instance = serializer.update(instance, serializer.validated_data)

            i+=1
        

class ApiTests(AgapeTests):

    def stringifyDatesAndTimes(self, data):
        for record in data:
            # for each item in create data
            for key, value in record.items():
                # if it is a datetime object
                if isinstance(value, datetime.datetime):
                    # format as a datetime string
                    record[key] = value.strftime('%Y-%m-%dT%H:%M:%S')
                 # if it is a date object
                elif isinstance(value,datetime.date):
                     # format as a date string
                    record[key] = value.strftime('%Y-%m-%d')

    def test_create(self):

        i=0
        while i < len(self.create_data):

            response = self.client.post(self.api_end_point, self.create_data[i])
            self.assertEqual(response.status_code, 201, "Created new instance")

            # compare response to expected data
            for key, expected_value in self.expect_data[i].items():
                instance_value = response.data.get(key)
                self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

            # verify actual database record was created
            instance = self.model.objects.get(id=response.data.get('id'))
            self.assertTrue(instance)

            i+=1

    def test_retrieve(self):

        i=0
        while i < len(self.create_data):

            # create the item to retrieve
            response = self.client.post(self.api_end_point, self.create_data[i])
            self.assertEqual(response.status_code, 201, "Created new instance")

            # remember the item id
            instance_id = response.data.get('id')
            response = None

            # retrieve the item via the api endpoint
            uri = "{}{}/".format(self.api_end_point,instance_id)
            response = self.client.get(uri)
            self.assertEqual(response.status_code, 200, "Retrieved")

            # compare response to expected data
            for key, expected_value in self.expect_data[i].items():
                instance_value = response.data.get(key)
                self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

            i+=1


    def test_update(self):

        i=0
        while i < len(self.create_data):

            # create the item to test
            response = self.client.post(self.api_end_point, self.create_data[i])
            self.assertEqual(response.status_code, 201, "Created new instance")

            # remember the item id
            instance_id = response.data.get('id')
            response = None

            # perform the update via the api endpoint
            uri = "{}{}/".format(self.api_end_point,instance_id)
            response = self.client.patch(uri, json.dumps(self.update_data[i]), content_type='application/json')
            self.assertEqual(response.status_code, 200, "Updated instance")

            # retrieve the item via the api endpoint
            response = None
            response = self.client.get(uri)

            # compare response to expected data
            for key, expected_value in self.update_expect_data[i].items():
                instance_value = response.data.get(key)
                self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )


            i+=1

    def test_delete(self):

        i=0
        while i < len(self.create_data):

            # create the item to test
            response = self.client.post(self.api_end_point, self.create_data[i])
            self.assertEqual(response.status_code, 201, "Created new instance")

            # remember the item id
            instance_id = response.data.get('id')
            response = None

            # perform delete via the api endpoint
            uri = "{}{}/".format(self.api_end_point,instance_id)
            response = self.client.delete( uri )    
            self.assertEqual(response.status_code, 204, "Deleted instance")

            # retrieve the item via the api endpoint
            response = None
            response = self.client.get(uri)
            self.assertEqual(response.status_code, 404, "Instance not found")


            i+=1

    def test_list(self):

        i=0
        while i < len(self.create_data):

            # create the item to test
            response = self.client.post(self.api_end_point, self.create_data[i])
            self.assertEqual(response.status_code, 201, "Created new instance")

            i+=1

        response = None
        response = self.client.get(self.api_end_point)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), len(self.create_data))         




from django.conf import settings
AUTHENTICATION = getattr(settings, 'AUTHENTICATION', {})
AUTHENTICATION['REQUIRE_ACTIVATION'] = AUTHENTICATION.get('REQUIRE_ACTIVATION', True)
AUTHENTICATION['RESET_PASSWORD_EXPIRES'] = AUTHENTICATION.get('RESET_PASSWORD_EXPIRES', 20)

# TODO: Implement
# AUTHENTICATION['OPEN_REGISTRATION'] = AUTHENTICATION.get('OPEN_REGISTRATION', True)
# AUTHENTICATION['USERNAMES'] = AUTHENTICATION.get('USERNAMES', False)



from rest_framework import serializers

from .models import Member
from agape.people.models import Person
from agape.people.serializers import PersonSerializer

class MemberSerializer(serializers.ModelSerializer):

	id = serializers.ReadOnlyField()
	person_id = serializers.IntegerField()
	join_date = serializers.DateTimeField(required=False)
	person = PersonSerializer(required=False)


	class Meta:
		model = Member

		fields = ('id', 'moniker', 'progenitor', 'person','person_id','role', 'involvement', 'join_date')

		read_only_fields = ['id','moniker','person']






from agape.signals import trigger
from rest_framework import permissions,status,views,viewsets
from rest_framework.response import Response



class ModelViewSet(viewsets.ModelViewSet):


	def create(self, request, *args, **kwargs):

		# emit signal
		trigger(self.context+'.create:request',request,*args,**kwargs)

		# get a copy of the request data
		data = request.data.copy()
		trigger(self.context+'.create:before',data)

		# create new instance
		serializer = self.get_serializer(data=data)
		serializer.is_valid(raise_exception=True)
		instance = self.perform_create(serializer)
		trigger(self.context+'.create:success',instance)
		
		# serialize instance data
		serializer = self.get_serializer(instance)
		serialized_data = serializer.data
		trigger(self.context+'.create:serialize',serialized_data)		

		# return a response
		headers = self.get_success_headers(serialized_data)
		response = Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)
		trigger(self.context+'.create:response',response)
		return response

	def perform_create(self, serializer):
		instance = serializer.save()
		return instance

	def retrieve(self, request, *args, **kwargs):
		trigger(self.context+'.retrieve:request',request,*args,**kwargs)

		trigger(self.context+'.retrieve:before',kwargs['pk'])

		instance = self.get_object()
		trigger(self.context+'.retrieve:success',instance)

		serializer = self.get_serializer(instance)
		data = serializer.data
		trigger(self.context+'.retrieve:serialize',data)

		response = Response(data)
		trigger(self.context+'.retrieve:response',response)
		return response

	def update(self, request, *args, **kwargs):
		trigger(self.context+'.update:request',request,*args,**kwargs)

		partial = kwargs.pop('partial', False)        
		instance = self.get_object()
		trigger(self.context+'.update:retrieve',instance)

		data = kwargs.get('data') or request.data
		trigger(self.context+'.update:before',data)

		serializer = self.get_serializer(instance, data=request.data, partial=partial)
		serializer.is_valid(raise_exception=True)
		instance = self.perform_update(serializer)
		trigger(self.context+'.update:success',instance)
        
		data = serializer.data
		trigger(self.context+'.update:serialize',data)	

		response = Response(data)
		trigger(self.context+'.update:response',response)
		return response

	def perform_update(self, serializer):
		instance = serializer.save()
		return instance

	def destroy(self, request, *args, **kwargs):

		trigger(self.context+'.destroy:request',request,*args,**kwargs)
		
		instance = self.get_object()
		trigger(self.context+'.destroy:before',instance)

		self.perform_destroy(instance)
		trigger(self.context+'.destroy:success',instance)

		response = Response(status=status.HTTP_204_NO_CONTENT)
		trigger(self.context+'.destroy:response',response)

		return response

	def list(self, request, *args, **kwargs):
		queryset = self.filter_queryset(self.get_queryset())

		page = self.paginate_queryset(queryset)
		if page is not None:
		    serializer = self.get_serializer(page, many=True)
		    return self.get_paginated_response(serializer.data)

		serializer = self.get_serializer(queryset, many=True)
		return Response(serializer.data)      

	def perform_destroy(self, instance):
		instance.delete()





from django.test import TestCase
from rest_framework.test import APIRequestFactory
from rest_framework.test import APIClient

from agape.signals import on

import json
from django.utils import timezone
from datetime import timedelta
import datetime

# Create your tests here.
from .models import Person
from .serializers import PersonSerializer
# from .settings import AUTHENTICATION
# 

MODEL = Person
SERIALIZER_CLASS = PersonSerializer
CREATE_DATA =  {   
        'first_name':'Elvis',
        'middle_name':'Aaron',
        'last_name':'Presley',
        'birthday': datetime.date(1935, 1, 8),
        'gender': 'm'
    }
EXPECT_DATA = {   
    'id':1,
    'first_name':'Elvis',
    'middle_name':'Aaron',
    'last_name':'Presley',
    'birthday': datetime.date(1935, 1, 8),
    'gender': 'm',
    'title': None
}
SERIALIZED_EXPECT_DATA = EXPECT_DATA.copy()
SERIALIZED_EXPECT_DATA['birthday'] = EXPECT_DATA['birthday'].strftime('%Y-%m-%d')

UPDATE_DATA = { 
    'title': 'Mr.'
}
UPDATE_EXPECT_DATA= {   
    'id':1,
    'first_name':'Elvis',
    'middle_name':'Aaron',
    'last_name':'Presley',
    'birthday': datetime.date(1935, 1, 8),
    'gender': 'm',
    'title': 'Mr.'
}
UPDATE_SERIALIZED_EXPECT_DATA = UPDATE_EXPECT_DATA.copy()
UPDATE_SERIALIZED_EXPECT_DATA['birthday'] = UPDATE_EXPECT_DATA['birthday'].strftime('%Y-%m-%d')


class PersonTestCase(TestCase):

    def setUp(self):
        self.model              = MODEL
        self.serializer_class   = SERIALIZER_CLASS
        self.create_data        = CREATE_DATA
        self.expect_data        = EXPECT_DATA
        self.update_data        = UPDATE_DATA
        self.update_expect_data = UPDATE_EXPECT_DATA

    def test_create(self):
        instance = self.model(**self.create_data)
        instance.save()

        # compare instance to expected data
        for key in self.expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_update(self):
        instance = self.model(**self.create_data)
        instance.save()

        # update the instance
        for key, value in self.update_data.items():
            instance_value = setattr(instance, key, value )
        
        # save the instance
        instance.save()

        # retrieve the instance
        instance_id = instance.id
        instance = None
        instance = self.model.objects.get(id=instance_id)

        # compare instance to expected data
        for key in self.update_expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.update_expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )



class PersonSerializerTestCase(TestCase):

    def setUp(self):
        self.model              = MODEL
        self.serializer_class   = SERIALIZER_CLASS
        self.create_data        = CREATE_DATA.copy()
        self.create_data['birthday'] ='1935-01-08'

        self.expect_data        = EXPECT_DATA.copy()

        self.instance_expect_data = EXPECT_DATA.copy()
        self.serialized_expect_data = SERIALIZED_EXPECT_DATA.copy()

        self.update_data        = UPDATE_DATA
        self.update_expect_data = UPDATE_SERIALIZED_EXPECT_DATA.copy()
        self.update_expect_data['birthday'] ='1935-01-08'

    def test_serialize(self):
        instance =self.model(**self.create_data)
        instance.save()

        serializer = self.serializer_class(instance)
        self.assertDictEqual(self.serialized_expect_data, serializer.data)

    def test_create(self):

        # serializer data is valid
        serializer = self.serializer_class(data=self.create_data)
        self.assertTrue(serializer.is_valid(), 'Serializer data is valid')

        # create an instance via the serializer
        instance = serializer.create(serializer.validated_data)
        self.assertTrue(instance, 'Created instance from serializer data')

        # compare instance to expected data
        for key in self.instance_expect_data.keys():
            instance_value = getattr(instance, key)

            # if instance value is date, convert to YMD
            expected_value = self.instance_expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_update(self):
        instance =self.model(**self.create_data)
        instance.save()

        serializer = self.serializer_class(instance,data=self.update_data,partial=True)
        self.assertTrue(serializer.is_valid(raise_exception=True), 'Serializer data is valid')

        instance = serializer.update(instance, serializer.validated_data)

        # compare instance to expected data
        for key, expected_value in self.update_expect_data.items():
            instance_value = getattr(instance, key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )



class APITestCase(TestCase):

    def setUp(self):
        self.model              = MODEL
        self.serializer_class   = SERIALIZER_CLASS
        self.create_data        = CREATE_DATA.copy()

        self.expect_data        = EXPECT_DATA.copy()

        self.instance_expect_data = EXPECT_DATA.copy()
        self.serialized_expect_data = SERIALIZED_EXPECT_DATA.copy()

        self.update_data        = UPDATE_DATA
        self.update_expect_data = UPDATE_SERIALIZED_EXPECT_DATA.copy()

        self.client =  APIClient()
        self.entity = 'person'
        self.api_end_point = '/api/v1/people/'

    def test_create(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        # compare response to expected data
        for key, expected_value in self.serialized_expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

        # verify actual database record was created
        instance = self.model.objects.get(id=response.data.get('id'))
        self.assertTrue(instance)

    def test_retrieve(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)
        self.assertEqual(response.status_code, 201, "Retrieved")

        response = None
        response = self.client.get(uri)

        # compare response to expected data
        for key, expected_value in self.serialized_expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_update(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = self.client.patch(uri, json.dumps(self.update_data), content_type='application/json')
        self.assertEqual(response.status_code, 200, "Updated instance")

        # validate response
        for key, expected_value in self.update_expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

        # perform get operation and validate
        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = None
        response = self.client.get(uri)

        # compare response to expected data
        for key, expected_value in self.update_expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_delete(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = self.client.delete( uri )    
        self.assertEqual(response.status_code, 204, "Deleted")

        query = self.model.objects.filter(id=instance_id)
        self.assertEqual(len(query),0, "Deleted")


    def test_list(self):

        # create people
        data = [
            {'title': 'Mr.', 'first_name':'Ben'     , 'last_name':'Grimm'   , 'birthday': '1980-01-01', 'gender':'m'},
            {'title': 'Mr.', 'first_name':'Johnny'  , 'last_name':'Storm'   , 'birthday': '1990-07-11', 'gender':'m'},
            {'title': 'Miss', 'first_name':'Sue'    , 'last_name':'Storm'   , 'birthday': '1989-12-28', 'gender':'f'},
            {'title': 'Dr.', 'first_name':'Reed'    , 'last_name':'Richards', 'birthday': '1969-04-11', 'gender':'m'}
        ]
        for idata in data:
            person = Person.objects.create(**idata )

        # list results
        response = None
        response = self.client.get(self.api_end_point)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 4)

        # filter - birthday exact
        response = None
        response = self.client.get(self.api_end_point+'?birthday=1980-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1)

        # filter - birthday_gt
        response = None
        response = self.client.get(self.api_end_point+'?birthday__gt=1985-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 2)        

        response = None
        response = self.client.get(self.api_end_point+'?birthday__gt=2000-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 0) 

        # filter - birthday_lt
        response = None
        response = self.client.get(self.api_end_point+'?birthday__lt=1985-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 2)        

        response = None
        response = self.client.get(self.api_end_point+'?birthday__lt=1969-04-11')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 0)    

        # filter - title
        response = None
        response = self.client.get(self.api_end_point+'?title=Mr.')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 2)   

        response = None
        response = self.client.get(self.api_end_point+'?title=Dr.')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1)

        response = None
        response = self.client.get(self.api_end_point+'?title=Miss')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1)                     

        # filter - gender
        response = None
        response = self.client.get(self.api_end_point+'?gender=m')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 3)   

        response = None
        response = self.client.get(self.api_end_point+'?gender=f')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1)

        # filter - first_name
        response = None
        response = self.client.get(self.api_end_point+'?first_name=Ben')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1) 

        response = None
        response = self.client.get(self.api_end_point+'?first_name=ben')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1) 

        # filter - last_name
        response = None
        response = self.client.get(self.api_end_point+'?last_name=Grimm')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1) 

        response = None
        response = self.client.get(self.api_end_point+'?last_name=Storm')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 2) 

        response = None
        response = self.client.get(self.api_end_point+'?last_name=storm')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 2) 


    def test_signals(self):

        scope = {}

        def on_create_before(self,data):
            scope['on_create_before'] = True

        def on_create_success(self,instance):
            scope['on_create_success'] = True

        def on_retrieve_before(self,data):
            scope['on_retrieve_before'] = True

        def on_retrieve_success(self,instance):
            scope['on_retrieve_success'] = True

        def on_update_before(self,data):
            scope['on_update_before'] = True

        def on_update_success(self,instance):
            scope['on_update_success'] = True

        def on_destroy_before(self,data):
            scope['on_destroy_before'] = True

        def on_destroy_success(self,instance):
            scope['on_destroy_success'] = True

        on(self.entity + '.create:before',      on_create_before)
        on(self.entity + '.create:success',     on_create_success)
        on(self.entity + '.retrieve:before',    on_retrieve_before)
        on(self.entity + '.retrieve:success',   on_retrieve_success)
        on(self.entity + '.update:before',      on_update_before)
        on(self.entity + '.update:success',     on_update_success)
        on(self.entity + '.destroy:before',      on_destroy_before)
        on(self.entity + '.destroy:success',     on_destroy_success)

        # create new record
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        self.assertTrue(scope['on_create_before'], '.create:before')
        self.assertTrue(scope['on_create_success'], '.create:before')

        # rerieve the record
        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)
        response = self.client.get(uri)
        self.assertEqual(response.status_code, 200, "Retrieved")

        self.assertTrue(scope['on_retrieve_before'], '.retrieve:before')
        self.assertTrue(scope['on_retrieve_success'], '.retrieve:success')


        # update a record
        response = self.client.patch(uri, json.dumps(self.update_data), content_type='application/json')
        self.assertEqual(response.status_code, 200, "Updated instance")

        self.assertTrue(scope['on_update_before'], '.update:before')
        self.assertTrue(scope['on_update_success'], '.update:before')

        # destroy
        response = self.client.delete( uri )    
        self.assertEqual(response.status_code, 204, "Destroyd")

        self.assertTrue(scope['on_destroy_before'], '.destroy:before')
        self.assertTrue(scope['on_destroy_success'], '.destroy:before')
       





from agape.signals import on
from .models import File
from .serializers import FileSerializer

class FilesConnector():

	app = 'agape.files'


	def to(self,target_app):
		""" Connect the files application to another application.
			
			connect = FilesConnector()
			connect.to('agape.people')
		"""

		self.connect(target_app)

	def connect(self,target_app):
		if target_app == 'agape.groups':
			self.connect_to_entity('group')

	def connect_to_entity(self, entity):
		self.connect_to_delete_instance(entity)

	def connect_to_delete_instance(entity):

    	# scope variable is shared between callbacks
		scope = {}

		# get the moniker of the entity being deleted
		def before(o,instance):
			scope['query'] = Files.objects.filter(progenitor=instance.moniker())

		# delete all files associated with the moniker
		def delete_files(o,instance):
			scope['query'].delete()

		# attach signals
		on(entity+'.delete:before',before)
		on(entity+'.delete:success',delete_files)





from django.db import models


# Create your models here.
class Model(models.Model):

    entity = 'undefined'

    def moniker(self):
        return '{}:{}'.format(self.entity, self.id)

    def __str__(self):
        return '<{}>'.format(self.moniker())

    class Meta:
        abstract = True


from rest_framework import serializers

from .models import File

class FileSerializer(serializers.ModelSerializer):

	id = serializers.ReadOnlyField()

	class Meta:
		model = File

		fields = ('id', 'progenitor', 'name', 'type' , 'path', 'label', 'description')

		read_only_fields = ['id']


from django.contrib.auth import update_session_auth_hash

from rest_framework import serializers

from django.contrib.auth import get_user_model

from .models import UserManager

class UserSerializer(serializers.ModelSerializer):
    password = serializers.CharField(write_only=True, required=False)
    confirm_password = serializers.CharField(write_only=True, required=False)

    class Meta:
        User = get_user_model()
        model = User
        fields = ('id', 'email', 'password', 'confirm_password', 'status')
        read_only_fields = ['id']

    def create(self, validated_data):

        User = get_user_model()
        password = validated_data.get('password', None)
        confirm_password = validated_data.get('confirm_password', None)

        # create the instance
        instance = User(email=validated_data['email'],status=validated_data['status'])

        # set the password
        instance = self.set_password(instance, password, confirm_password)

        # save
        instance.save()            

        return instance

    def update(self, instance, validated_data):
        instance.email = validated_data.get('email', instance.email)
        password = validated_data.get('password', None)

        # set the password
        instance = self.set_password(instance,password,password)

        # save
        instance.save()  

        update_session_auth_hash(self.context.get('request'), instance)

        return instance

    def set_password(self, instance, password, confirm_password):
        # if confirm password supplied
        if confirm_password:

            # test that password and confirmation match
            if password and confirm_password and password == confirm_password:
                instance.set_password(password)

            # password/confirmation do not match
            else:
                raise ValueError('Passwords do not match.')

        # if no password confirmation, just save the password
        elif password:
            instance.set_password(password)

        return instance
        



from agape import viewsets

from .models import Person, PersonFilter
from .serializers import PersonSerializer

from django_filters import rest_framework as filters


class PersonViewSet(viewsets.ModelViewSet):
	""" Viewset that provides CRUD operations for people.

	Extends:
		viewset.ModelViewSet

	"""

	queryset = Person.objects.all()
	serializer_class = PersonSerializer
	context = 'person'

	filter_backends = (filters.DjangoFilterBackend,)
	filter_class = PersonFilter




from agape.signals import on
from .models import Member
from .serializers import MemberSerializer


class MembersConnector():

	app = 'agape.members'
	model = Member


	def to(self,target_app):
		self.connect(target_app)


	def connect(self,target_app):

		if target_app == 'agape.organizations':
			self.connnect_to_entity('organizations')

		elif target_app == 'agape.groups':
			self.connect_to_entity('groups')


	def connect_to_entity(self, entity):
		self.connect_to_create_instance(entity)


	def connect_to_create_instance(self, entity):
		""" Create new contact records when creating a new instance of given entity.
		"""

		# scope variable is shared between callbacks
		scope = {}

		# catch the incoming data to use after entity instance is created
		def catch_incoming_data(o,data):
			scope = {}
			scope['memberData'] = None

			if 'members' in data:
				scope['membertData'] = data.pop('members',[])

		# create contacts from given data and associate with entity instance
		def create_members(o,instance):

			members = []

			for personId in scope['memberData']:
				member = { 'progenitor':instance.moniker(), 'person':personId}
				members.append(member)

			serializer = MemberSerializer(data=members, many=True)
			if serializer.is_valid():
			    recordSet = serializer.save()

		# register callbacks
		on(entity+'.create:before',catch_incoming_data)
		on(entity+'.create:success',create_contacts) 

	def connect_to_delete_instance(entity):

    	# scope variable is shared between callbacks
		scope = {}

		def before(o,instance):
			scope['query'] = Member.objects.filter(progenitor=instance.moniker())

		def delete_children(o,instance):
			scope['query'].delete()

		on(entity+'.delete:before',before)
		on(entity+'.delete:success',delete_children)





from django.conf.urls import url, include
from .views import PersonViewSet

# build router
from rest_framework.routers import DefaultRouter

router = DefaultRouter()
router.register(r'people', PersonViewSet, base_name='people')

urlpatterns = [
    url(r'^',include(router.urls)),
]


from rest_framework import permissions,status,views,viewsets
from rest_framework.response import Response
from rest_framework import generics

from .models import File
from .serializers import FileSerializer


from agape.signals import trigger
from django.core.files.storage import FileSystemStorage
import mimetypes
import json

class FileViewSet(viewsets.ModelViewSet):
    """ Viewset that provides CRUD operations for people.

    Extends:
        viewset.ModelViewSet

    """

    queryset = File.objects.all()
    serializer_class = FileSerializer
    context = 'files'



    def create(self,request,*args,**kwargs):
        """ Perform file upload.
            
            Accepts multi-part form data to handle file uploads.
        """
        # trigger(self.context+'.create:request',request,*args,**kwargs)

        # data = request.data.copy()
        # trigger(self.context+'.create:before',data)

        # check for the existence of the file key in the request
        if not 'file' in request.FILES:

            # throw error if no file field supplied
            message  = {'status':'fail', 'message': 'No field matching name "file" is present.'}
            response = Response({'status':'fail'}, status=status.HTTP_400_BAD_REQUEST)
            trigger(self.context+'.create:fail',response)
            return response            

        # store the file to the file system
        file = request.FILES['file']

        fs = FileSystemStorage()
        filename = fs.save(file.name, file)
        uploaded_file_url = fs.url(filename)

        # store the relative path to the file on disk
        request.data['path'] = filename;

        # call the parent create method
        return super().create(request,*args,**kwargs)




class FileDownloadView(views.APIView):

    context = 'file'

    def retrieve(self,request,*args,**kwargs):

        print('GET FILE')

        kwargs['pk']=1

        trigger(self.context+'.download:request',request,*args,**kwargs)

        trigger(self.context+'.download:before',kwargs['pk'])

        instance = File.objects.get(pk=kwargs['pk'])
        mimetype = mimetypes.guess_type(instance.full_path())

        # open the file
        filepointer = open(instance.full_path(),"r")

        # stream file to http response
        response = HttpResponse(filepointer,content_type=mimetype)
        response['Content-Disposition'] = 'attachment; filename={}'.format( instance.name )


        return response






from django.db import models

# Create your models here.
class Person (models.Model):

    entity = 'person'

    FEMALE = 'f'
    MALE = 'm'
    GENDER_CHOICES = (
        ("", '-- select --'),
        (FEMALE, 'Female'),
        (MALE, 'Male')
    )
    
    progenitor  = models.CharField(max_length=64,null=True,blank=True)
    title       = models.CharField(max_length=16,null=True,blank=True)

    first_name  = models.CharField(max_length=32,null=False,blank=False)
    middle_name = models.CharField(max_length=32,null=True ,blank=True)
    last_name   = models.CharField(max_length=32,null=True ,blank=True)
    
    gender      = models.CharField(max_length=1,null=True,blank=True,choices=GENDER_CHOICES)
    birthday    = models.DateField(blank=True,null=True)

    def moniker(self):
        return '{}:{}'.format(self.entity, self.id)

    def __str__(self):
        return '<{} {} {}>'.format(self.moniker(), self.first_name, self.last_name)



from datetime import datetime
from django_filters import rest_framework as filters


class PersonFilter(filters.FilterSet):

    first_name  = filters.CharFilter(lookup_expr='icontains')
    middle_name = filters.CharFilter(lookup_expr='icontains')
    last_name   = filters.CharFilter(lookup_expr='icontains')


    class Meta:
        model = Person
        fields = {
            'progenitor': ['exact'],
            'title': ['exact'],
            'gender': ['exact'],
            'birthday': ['exact','lt','gt']
        }


from rest_framework import serializers

from .models import Contact

class ContactSerializer(serializers.ModelSerializer):

	id = serializers.ReadOnlyField()

	class Meta:
		model = Contact

		fields = ('id', 'progenitor', 'label', 'type', 'value' )

		read_only_fields = ['id']


from django.db import models
from django.template.defaultfilters import slugify

import re
import itertools

# Create your models here.
class Organization (models.Model):

    entity = 'organization'

    prepopulated_fields = {"slug": ("name")}
   
    progenitor  = models.CharField(max_length=64,null=True,blank=True)
    description = models.TextField(null=True,blank=True)
    name        = models.CharField(max_length=255,null=False,blank=False)
    slug        = models.SlugField(max_length=255,allow_unicode=True,blank=True)

    class Meta:
        permissions = (
            ('view_organization', 'View organization'),
        )

    def moniker(self):
        return '{}:{}'.format(self.entity, self.id)

    def save(self, *args, **kwargs):
        self.set_slug_value(self.name)
        # save
        super().save(*args, **kwargs)


    def set_slug_value(self,slugbase):
       
        # check for user supplied slug
        if self.slug:
            # make sure it is really a slug
            self.slug = original = slugify(self.slug)

            # check for integer suffix, only perform this on user supplied
            # slugs to allow users to enter names like "Movie Sequel 2" that
            # will be duplicated as "movie-sequel-2-2" instead of "movie-sequel-3"
            result = re.search(r'-(\d)+$', self.slug)

            if result:
                base = re.sub(r'-(\d)+$', '', self.slug)
                integer = int(result.group(1))
            else:
                base = self.slug
                integer = 1

        # create a slug from the organization name
        else:
            self.slug = original = slugify(slugbase)
            base = self.slug
            integer = 1

        # find maximum length of the slug field
        max_length = self.__class__._meta.get_field('slug').max_length

        # ensure uniquness of the slug field
        # TODO: Add setting to enable slug uniqueness based only on progenitor
        for x in itertools.count(integer+1):
             # do not include this instance when checking for existing slug with this value
            query = self.__class__.objects.filter(slug=self.slug)
            query = query.exclude(id=self.id)

            # no slug with this name found, continue
            if not query.exists():
                break

            # truncate the slug to fit maximum length of field
            self.slug = "%s-%d" % (base[:max_length - len(str(x)) - 1], x)




    def __str__(self):
        return '<{} {}>'.format(self.moniker(), self.name)



from django.contrib import admin

# Register your models here.
from django.contrib import admin
from .models import User,UserActivation,ResetPasswordRequest

admin.site.register(User)
admin.site.register(UserActivation)
admin.site.register(ResetPasswordRequest)



# testing classes
from django.test import TestCase
from rest_framework.test import APIRequestFactory
from rest_framework.test import APIClient

# agape classes
from agape.signals import on

# utility classes
import json
from django.utils import timezone
from datetime import timedelta
import datetime

# package classes
from .models import Member
from .serializers import MemberSerializer
from agape.people.models import Person
from agape.organizations.models import Organization


MODEL = Member
SERIALIZER_CLASS = MemberSerializer
CREATE_DATA = {
    'progenitor':'organization:1',
    'person_id':'1'
}

EXPECT_DATA = {   
    'id':1,
    'progenitor':'organization:1'
}

UPDATE_DATA = { 
    'join_date': '2001-01-01 00:00:00'
}
UPDATE_EXPECT_DATA= {   
    'join_date':  datetime.datetime(2001, 1, 1, 0, 0)
}

class ModelTestCase(TestCase):

    def setUp(self):
        self.model              = MODEL
        self.serializer_class   = SERIALIZER_CLASS
        self.create_data        = CREATE_DATA
        self.expect_data        = EXPECT_DATA
        self.update_data        = UPDATE_DATA
        self.update_expect_data = UPDATE_EXPECT_DATA

        self.person = Person.objects.create(first_name="Martin",last_name="Luther")

    def test_create(self):
        instance = self.model(**self.create_data)
        instance.save()
        self.assertEqual(instance.id, 1, "Instance saved to database" )


        # compare instance to expected data
        for key in self.expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

        member = self.model.objects.get(pk=instance.id)
        self.assertTrue(member,"Retrieved member object")
        self.assertTrue(member.person,"Rtreieved person object")


    def test_update(self):
        instance = self.model(**self.create_data)
        instance.save()

        # update the instance
        for key, value in self.update_data.items():
            instance_value = setattr(instance, key, value )
        
        # save the instance
        instance.save()

        # retrieve the instance
        instance_id = instance.id
        instance = None
        instance = self.model.objects.get(id=instance_id)

        # compare instance to expected data
        for key in self.update_expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.update_expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )



class SerializerTestCase(TestCase):

    def setUp(self):
        self.model              = MODEL
        self.serializer_class   = SERIALIZER_CLASS
        self.create_data        = CREATE_DATA
        self.expect_data        = EXPECT_DATA
        self.update_data        = UPDATE_DATA
        self.update_expect_data = UPDATE_EXPECT_DATA

        self.person = Person.objects.create(first_name="Martin",last_name="Luther")

    def test_serialize(self):
        instance =self.model(**self.create_data)
        instance.save()

        serializer = self.serializer_class(instance)

        # compare instance to expected data
        for key in self.expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_create(self):

        # serializer data is valid
        serializer = self.serializer_class(data=self.create_data)
        self.assertTrue(serializer.is_valid(), 'Serializer data is valid')

        # create an instance via the serializer
        instance = serializer.create(serializer.validated_data)
        self.assertTrue(instance, 'Created instance from serializer data')

        # compare instance to expected data
        for key in self.expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_update(self):
        instance =self.model(**self.create_data)
        instance.save()


        serializer = self.serializer_class(data=self.update_data,partial=True)
        self.assertTrue(serializer.is_valid(), 'Serializer data is valid')
        instance = serializer.update(instance, serializer.validated_data)

        # compare instance to expected data
        for key, expected_value in self.update_expect_data.items():
            instance_value = getattr(instance, key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )



class APITestCase(TestCase):

    def setUp(self):
        self.model              = MODEL
        self.serializer_class   = SERIALIZER_CLASS
        self.create_data        = CREATE_DATA
        self.expect_data        = EXPECT_DATA
        self.update_data        = UPDATE_DATA
        self.update_expect_data = UPDATE_EXPECT_DATA.copy()
        self.update_expect_data['join_date'] = '2001-01-01T00:00:00'

        self.client =  APIClient()
        self.entity = 'member'
        self.api_end_point = '/api/v1/members/'

    def test_create(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        # compare response to expected data
        for key, expected_value in self.expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

        # verify actual database record was created
        instance = self.model.objects.get(id=response.data.get('id'))
        self.assertTrue(instance)

    def test_retrieve(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)
        self.assertEqual(response.status_code, 201, "Retrieved")

        response = None
        response = self.client.get(uri)

        # compare response to expected data
        for key, expected_value in self.expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_update(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = self.client.patch(uri, json.dumps(self.update_data), content_type='application/json')
        self.assertEqual(response.status_code, 200, "Updated instance")

        # validate response
        for key, expected_value in self.update_expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

        # perform get operation and validate
        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = None
        response = self.client.get(uri)

        # compare response to expected data
        for key, expected_value in self.update_expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_delete(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = self.client.delete( uri )    
        self.assertEqual(response.status_code, 204, "Deleted")

        query = self.model.objects.filter(id=instance_id)
        self.assertEqual(len(query),0, "Deleted")

    def test_list(self):

        # create an organization
        organization = Organization.objects.create(name="Planet Express")

        # create people
        data = [
            {'title': 'Mr.', 'first_name':'Ben'     , 'last_name':'Grimm'   , 'birthday': '1980-01-01', 'gender':'m'},
            {'title': 'Mr.', 'first_name':'Johnny'  , 'last_name':'Storm'   , 'birthday': '1990-07-11', 'gender':'m'},
            {'title': 'Miss', 'first_name':'Sue'    , 'last_name':'Storm'   , 'birthday': '1989-12-28', 'gender':'f'},
            {'title': 'Dr.', 'first_name':'Reed'    , 'last_name':'Richards', 'birthday': '1969-04-11', 'gender':'m'}
        ]
        for idata in data:
            person = Person.objects.create(**idata )
            member = Member.objects.create(progenitor=organization.moniker(),person=person,join_date='2001-01-01')
        
        # list results
        response = None
        response = self.client.get(self.api_end_point)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 4)

        # filter - progenitor
        response = None
        response = self.client.get(self.api_end_point+'?progenitor=organization:1')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 4)

        # filter - join date exact
        response = None
        response = self.client.get(self.api_end_point+'?join_date=2001-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 4)

        # filter - join date gt
        response = None
        response = self.client.get(self.api_end_point+'?join_date__gt=2001-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 0)        

        response = None
        response = self.client.get(self.api_end_point+'?join_date__gt=2000-12-31')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 4) 

        # filter - join date lt
        response = None
        response = self.client.get(self.api_end_point+'?join_date__lt=2001-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 0)        

        response = None
        response = self.client.get(self.api_end_point+'?join_date__lt=2000-01-02')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 4)   

        # filter - birthday exact
        response = None
        response = self.client.get(self.api_end_point+'?person__birthday=1980-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1)

        # filter - birthday_gt
        response = None
        response = self.client.get(self.api_end_point+'?person__birthday__gt=1985-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 2)        

        response = None
        response = self.client.get(self.api_end_point+'?person__birthday__gt=2000-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 0) 

        # filter - birthday_lt
        response = None
        response = self.client.get(self.api_end_point+'?person__birthday__lt=1985-01-01')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 2)        

        response = None
        response = self.client.get(self.api_end_point+'?person__birthday__lt=1969-04-11')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 0)    

        # filter - title
        response = None
        response = self.client.get(self.api_end_point+'?person__title=Mr.')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 2)   

        response = None
        response = self.client.get(self.api_end_point+'?person__title=Dr.')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1)

        response = None
        response = self.client.get(self.api_end_point+'?person__title=Miss')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1)                     

        # filter - gender
        response = None
        response = self.client.get(self.api_end_point+'?person__gender=m')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 3)   

        response = None
        response = self.client.get(self.api_end_point+'?person__gender=f')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1)

        # filter - first_name
        response = None
        response = self.client.get(self.api_end_point+'?person__first_name=Ben')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1) 

        response = None
        response = self.client.get(self.api_end_point+'?person__first_name=ben')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1) 

        # filter - last_name
        response = None
        response = self.client.get(self.api_end_point+'?person__last_name=Grimm')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 1) 

        response = None
        response = self.client.get(self.api_end_point+'?person__last_name=Storm')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 2) 

        response = None
        response = self.client.get(self.api_end_point+'?person__last_name=storm')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), 2)      





from rest_framework import permissions

class IsAccountOwner(permissions.BasePermission):
	
    def has_object_permissions(self,request,view,account):
        if request.user:
            return account == request.user
        return False
    



from django.test import TestCase
from rest_framework.test import APIRequestFactory
from rest_framework.test import APIClient

import json
from django.utils import timezone
from datetime import timedelta

# Create your tests here.
from .models import Organization
from .serializers import OrganizationSerializer


class OrganizationTestCase(TestCase):

    def setUp(self):
        self.create_data = { 'name': 'Umbrella Corporation' }
        self.expect_data = { 'id': 1, 'name': 'Umbrella Corporation', 'slug':'umbrella-corporation'}
        
        self.update_data = { 'name': 'Umbrella Foundation'}
        self.update_expect_data = {'name': 'Umbrella Foundation'}

        self.model = Organization
        self.serializer_class = OrganizationSerializer

    def test_create(self):
        instance = self.model(**self.create_data)
        instance.save()

        # compare instance to expected data
        for key in self.expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_update(self):
        instance = self.model(**self.create_data)
        instance.save()

        # update the instance
        for key, value in self.update_data.items():
            instance_value = setattr(instance, key, value )
        
        # save the instance
        instance.save()

        # retrieve the instance
        instance_id = instance.id
        instance = None
        instance = self.model.objects.get(id=instance_id)

        # compare instance to expected data
        for key in self.update_expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.update_expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_slug_user_supplied(self):
        instance = self.model(name="Umbrella Corporation", slug="umbrella-corp")
        instance.save()

        # test slug
        self.assertEqual(instance.slug, "umbrella-corp")

        # incorrectly formatted slug
        instance = Organization(name="First World Charity", slug="first world charity")
        instance.save()

        # test slug
        self.assertEqual(instance.slug, "first-world-charity")        

    def test_slug_duplicates(self):
        instance = Organization(name="Umbrella Corporation")
        instance.save()

        # first slug
        self.assertEqual(instance.slug, "umbrella-corporation")

        # second slug
        duplicate = Organization(name="Umbrella Corporation")
        duplicate.save()

        self.assertEqual(duplicate.slug, "umbrella-corporation-2")

        # third duplicate with correct integer suffix
        duplicate = Organization(name="Umbrella Corporation", slug="umbrella-corporation-2")
        duplicate.save()

        self.assertEqual(duplicate.slug, "umbrella-corporation-3")

    def test_slug_no_duplicate_renamed_on_update(self):
        """ ensure that when updating an object the slug is not falsely
        identified as a duplicate. """

        instance = Organization(name="Umbrella Corporation")
        instance.save()
        self.assertEqual(instance.slug, "umbrella-corporation")

        instance.save()
        self.assertEqual(instance.slug, "umbrella-corporation")



class OrganizationSerializerTestCase(TestCase):

    def setUp(self):
        self.create_data = { 'name': 'Umbrella Corporation' }
        self.expect_data = { 'id': 1, 'name': 'Umbrella Corporation', 'slug':'umbrella-corporation'}
        
        self.update_data = { 'name': 'Umbrella Foundation'}
        self.update_expect_data = {'name': 'Umbrella Foundation'}

        self.model = Organization
        self.serializer_class = OrganizationSerializer

    def test_serialize(self):
        instance =self.model(**self.create_data)
        instance.save()

        serializer = self.serializer_class(instance)
        self.assertDictEqual(self.expect_data, serializer.data)

    def test_create(self):

        # serializer data is valid
        serializer = self.serializer_class(data=self.create_data)
        self.assertTrue(serializer.is_valid(), 'Serializer data is valid')

        # create an instance via the serializer
        instance = serializer.create(serializer.validated_data)
        self.assertTrue(instance, 'Created instance from serializer data')

        # compare instance to expected data
        for key in self.expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_update(self):
        instance =self.model(**self.create_data)
        instance.save()

        serializer = self.serializer_class(data=self.update_data)
        self.assertTrue(serializer.is_valid(), 'Serializer data is valid')

        instance = serializer.update(instance, serializer.validated_data)

        # compare instance to expected data
        for key, expected_value in self.update_expect_data.items():
            instance_value = getattr(instance, key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )



             
class APITestCase(TestCase):

    def setUp(self):
        self.client =  APIClient()
        self.api_end_point = '/api/v1/organizations/'
        self.create_data = { 'name': 'Umbrella Corporation' }
        self.expect_data = { 'id': 1, 'name': 'Umbrella Corporation', 'slug':'umbrella-corporation'}
        
        self.update_data = { 'name': 'Umbrella Foundation'}
        self.update_expect_data = {'name': 'Umbrella Foundation'}

        self.model = Organization
        self.serializer_class = OrganizationSerializer

    def test_create(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        # compare response to expected data
        for key, expected_value in self.expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

        # verify actual database record was created
        instance = self.model.objects.get(id=response.data.get('id'))
        self.assertTrue(instance)

    def test_retrieve(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)
        self.assertEqual(response.status_code, 201, "Retrieved")

        response = None
        response = self.client.get(uri)

        # compare response to expected data
        for key, expected_value in self.expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_update(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = self.client.patch(uri, json.dumps(self.update_data), content_type='application/json')
        self.assertEqual(response.status_code, 200, "Updated instance")

        # validate response
        for key, expected_value in self.update_expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

        # perform get operation and validate
        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = None
        response = self.client.get(uri)

        # compare response to expected data
        for key, expected_value in self.update_expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_delete(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = self.client.delete( uri )    
        self.assertEqual(response.status_code, 204, "Deleted")

        query = self.model.objects.filter(id=instance_id)
        self.assertEqual(len(query),0, "Deleted")







from agape import viewsets

from .models import Member, MemberFilter
from .serializers import MemberSerializer

from django_filters import rest_framework as filters


class MemberViewSet(viewsets.ModelViewSet):
	""" Viewset that provides CRUD operations for people.

	Extends:
		viewset.ModelViewSet

	"""

	queryset = Member.objects.all()
	serializer_class = MemberSerializer
	context = 'member'

	filter_backends = (filters.DjangoFilterBackend,)
	filter_class = MemberFilter





from django.db import models


from django.utils import timezone
from agape.people.models import Person

# Create your models here.
class Member (models.Model):

    entity = 'member'

    progenitor      = models.CharField(max_length=64,null=True,blank=True)
    person          = models.ForeignKey(Person, on_delete=models.CASCADE, blank=False, null=False)
    
    role            = models.CharField(max_length=16,null=False,blank=True,default='member')
    involvement     = models.CharField(max_length=16,null=False,blank=True)
    join_date       = models.DateTimeField(blank=True,null=True,default=timezone.now)
    added_by        = models.ForeignKey(Person, on_delete=models.SET_NULL, blank=True, null=True, related_name='GroupMembers_added')

    def moniker(self):
        return '{}:{}'.format(self.entity, self.id)

    def __str__(self):
        return '<{} {}>'.format(self.moniker(), self.name)



from datetime import datetime
from django_filters import rest_framework as filters

class MemberFilter(filters.FilterSet):

    join_date = filters.CharFilter(method='filter_join_date__exact')
    join_date__gt = filters.CharFilter(method='filter_join_date__gt')
    join_date__lt = filters.CharFilter(method='filter_join_date__lt')

    person__first_name  = filters.CharFilter(lookup_expr='icontains')
    person__middle_name = filters.CharFilter(lookup_expr='icontains')
    person__last_name   = filters.CharFilter(lookup_expr='icontains')

    class Meta:
        model = Member
        fields = {
            'progenitor': ['exact'],
            'role': ['exact'],
            'involvement': ['exact'],
            'person__progenitor': ['exact'],
            'person__title': ['exact'],
            'person__gender': ['exact'],
            'person__birthday': ['exact','lt','gt']
        }

    def filter_join_date__exact(self, queryset, name, value):
        date = datetime.strptime(value, '%Y-%m-%d')
        date_range = (
            # The start_date with the minimum possible time
            datetime.combine(date, datetime.min.time()),
            # The start_date with the maximum possible time
            datetime.combine(date, datetime.max.time())
        )
        return queryset.filter(join_date__range=date_range)

    def filter_join_date__gt(self, queryset, name, value):
        date = datetime.strptime(value, '%Y-%m-%d')
        return queryset.filter(join_date__gt=date)

    def filter_join_date__lt(self,queryset, name, value):
        date = datetime.strptime(value, '%Y-%m-%d')
        date = datetime.combine(date, datetime.min.time())
        return queryset.filter(join_date__gt=date)





from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
from django.db import models
from .settings import AUTHENTICATION

# for random string generation
import string
import random

# for dispatching emails after creating new activation and password reset links
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.core.mail import send_mail



class UserManager(BaseUserManager):
    """
    A custom user manager to deal with emails as unique identifiers for
    auth instead of usernames. This replaces the default "UserManager".
    """
    def create_user(self,email,password,**extra_fields):
        """
        Creates and saves a User with the given email and password.
        """
        if not email:
            raise ValueError('The email must be set.')
        email = self.normalize_email(email)
        user = User(email=email,**extra_fields)
        user.set_password(password)
        user.save()
        return user
    
    def create_superuser(self,email,password,**extra_fields):
        extra_fields.setdefault('is_superuser',True)
        extra_fields.setdefault('status',1)
        
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')
        
        return self.create_user(email,password,**extra_fields)


class User(AbstractBaseUser, PermissionsMixin):
    """
    A custom user class which uses emails as unique identifiers instead
    of usernames. This replaces the default "User" supplied by Django.
    """
    DISABLED = -1
    INACTIVE = 0
    ACTIVE = 1
    STATUS_CHOICES = (
        ("", '-- select --'),
        (DISABLED, 'Disabled'),
        (INACTIVE, 'Inactive'),
        (ACTIVE, 'Active')
    )
    
    email        = models.EmailField(unique=True)
    is_superuser = models.BooleanField(default=False,help_text="Designates whether user can log into the backend.")
    status      = models.SmallIntegerField(choices=STATUS_CHOICES,blank=False,null=False,default=0 )
    last_login  = models.DateTimeField(blank=True,null=True)
    
    created     = models.DateTimeField(auto_now_add=True,blank=True,null=True)
    created_by  = models.ForeignKey('User', on_delete=models.PROTECT, blank=True, null=True, related_name="created_user")
    modified    = models.DateTimeField(auto_now=True,blank=True,null=True)
    modified_by = models.ForeignKey('User', on_delete=models.PROTECT, blank=True, null=True, related_name="modified_user")    

    USERNAME_FIELD = 'email'
    objects = UserManager()
    
    @property
    def is_admin(self):
      return self.is_superuser
    
    @property
    def is_staff(self):
      return self.is_superuser

    @property
    def is_active(self):
        return True if self.status == 1 else False;
    
    def get_short_name(self):
        self.email;
    
    def get_long_name(self):
        self.email;
    
    def __unicode__(self):
        return self.email






class UserActivation(models.Model):
    """ Provide activation links for new users.
    
    Generates a random key which is used to activate a new user account. Emails can
    then be sent to users with a link containing the key. Once this link has been
    followed the account will be set to active.
    
    Extends:
        models.Model
    
    Variables:
        user (models.ForeignKey): User account to activate
        key  (models.StringField): Randomly generated string of eight uppercase letters and numbers
        created (models.DateTimeField): Time activation link was created
        activated (models.DateTimeField): Time account was activated
    """

    def generate_key():
        """Generate a random string of 8 uppercase letters and numbers

        Returns:
            string: Random string of 8 uppercase letters and numbers

        """
        return ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(8))

    user      = models.ForeignKey('User', on_delete=models.CASCADE, blank=False, null=False)    
    key       = models.CharField(max_length=8,unique=True,blank=False,null=False,default=generate_key)
    created   = models.DateTimeField(auto_now_add=True,blank=True,null=False)
    activated = models.DateTimeField(blank=True,null=True)





class ResetPasswordRequest(models.Model):
    """ Provide password reset links for new users.
    
    Generates a random key which is used to reset user account passwords. Emails can
    then be sent to users with a link containing the key. Once this link has been
    followed the user can set a new password.
    
    Extends:
        models.Model
    
    Variables:
        user (models.ForeignKey): User account to activate
        key  (models.StringField): Randomly generated string of 16 uppercase letters and numbers
        created (models.DateTimeField): Time reset link was created
        activated (models.DateTimeField): Time reset link was followed
    """

    def generate_key():
        """Generate a random string of 16 uppercase letters and numbers

        Returns:
            string: Random string of 16 uppercase letters and numbers

        """
        return ''.join(random.SystemRandom().choice(string.ascii_uppercase + string.digits) for _ in range(16))


    DISABLED = -1
    ENABLED = 1
    USED = 2
    STATUS_CHOICES = (
        ("", '-- select --'),
        (DISABLED, 'Disabled'),
        (ENABLED, 'Active'),
        (USED, 'Used'),
    )

    user      = models.ForeignKey('User', on_delete=models.CASCADE, blank=False, null=False) 
    key       = models.CharField(max_length=16,unique=True,blank=False,null=False,default=generate_key)
    created   = models.DateTimeField(auto_now_add=True,blank=True,null=False)
    status    = models.SmallIntegerField(choices=STATUS_CHOICES,blank=False,null=False,default=1)
    used      = models.DateTimeField(blank=True,null=True)



from django.conf.urls import url, include
from .views import FileViewSet

# build router
from rest_framework.routers import DefaultRouter
from .views import FileDownloadView

router = DefaultRouter()
router.register(r'files', FileViewSet, base_name='files')

urlpatterns = [
    url(r'^',include(router.urls)),
    url(r'^view/$',FileDownloadView.as_view(), name='file-download')
]




from django.db import models

from django.contrib.auth import get_user_model
from django.dispatch import receiver
User = get_user_model()

from django.conf import settings
import os;


class File (models.Model):

    entity = 'file'
    
    progenitor  = models.CharField(max_length=64 ,null=True, blank=True)

    name         = models.CharField(max_length=255,null=True ,blank=True)
    type         = models.CharField(max_length=16 ,null=True ,blank=True)
    path         = models.TextField(max_length=255, null=True, blank=True)

    label        = models.CharField(max_length=255 ,null=True ,blank=True)
    description  = models.TextField(null=True,blank=True)

    uploaded     = models.DateTimeField(auto_now_add=True,blank=True,null=True)
    uploaded_by  = models.ForeignKey(User, on_delete=models.PROTECT, blank=True, null=True, related_name="uploaded_user")

    def full_path(self):
        return settings.MEDIA_ROOT + self.path;

    def moniker(self):
        return '{}:{}'.format(self.entity, self.id)

    def __str__(self):
        return '<{} {} {}>'.format(self.moniker(), self.type, self.value)


@receiver(models.signals.post_delete, sender=File)
def auto_delete_file_on_delete(sender, instance, **kwargs):
    """
    Deletes file from file system when the
    when corresponding `File` object is deleted.
    """
    if instance.full_path():
        if os.path.isfile(instance.full_path()):
            os.remove(instance.full_path())


@receiver(models.signals.pre_save, sender=File)
def auto_delete_file_on_change(sender, instance, **kwargs):
    """
    Deletes old file from filesystem
    when corresponding `File` object is updated
    with new file.
    """

    # don't check new files
    if not instance.pk:
        return False

    # get the old file path
    try:
        old_instance =  File.objects.get(pk=instance.pk)
    except File.DoesNotExist:
        return False

    # if the new file path and the old file path are not the same
    if not old_instance.path == instance.path:
        if os.path.isfile(old_instance.full_path()):
            os.remove(old_instance.full_path())



from django.db import models

# Create your models here.
class Contact (models.Model):

    entity = 'contact'

    TELEPHONE = 'Telephone'
    EMAIL     = 'Email'
    CHAT      = 'Chat'
    WEBSITE   = 'Website'
    PROFILE   = 'Profile'

    TYPE_CHOICES = (
        ("", '-- select --'),
        (TELEPHONE, TELEPHONE),
        (EMAIL, EMAIL),
        (CHAT, CHAT),
        (PROFILE, PROFILE),
        (WEBSITE, WEBSITE)
    )
    
    progenitor  = models.CharField(max_length=64 ,null=True, blank=True)
    type        = models.CharField(max_length=16 ,null=True ,blank=True,choices=TYPE_CHOICES)
    value       = models.CharField(max_length=255,null=True ,blank=True)
    label       = models.CharField(max_length=32 ,null=True ,blank=True)

    def moniker(self):
        return '{}:{}'.format(self.entity, self.id)

    def __str__(self):
        return '<{} {} {}>'.format(self.moniker(), self.type, self.value)



from rest_framework import serializers

from .models import Organization

class OrganizationSerializer(serializers.ModelSerializer):

	id = serializers.ReadOnlyField()

	class Meta:
		model = Organization

		fields = ('id', 'name','slug')

		read_only_fields = ['id']




# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-04 11:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('people', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Member',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('progenitor', models.CharField(blank=True, max_length=64, null=True)),
                ('role', models.CharField(blank=True, default='member', max_length=16)),
                ('involvement', models.CharField(blank=True, max_length=16)),
                ('join_date', models.DateTimeField(blank=True, default=django.utils.timezone.now, null=True)),
                ('added_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='GroupMembers_added', to='people.Person')),
                ('person', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='people.Person')),
            ],
        ),
    ]




from agape.signals import on
from .models import Contact
from .serializers import ContactSerializer

class ContactsConnector():

	app = 'agape.contacts'


	def to(self,target_app):
		""" Connect the contacts application to another application.
			
			connect = ContactsConnector()
			connect.to('agape.people')
		"""

		self.connect(target_app)

	def connect(self,target_app):
		if target_app == 'agape.people':
			self.connect_to_entity('person')

		elif target_app == 'agape.organizations':
			self.connnect_to_entity('organizations')


	def connect_to_entity(self, entity):
		self.connect_to_create_instance(entity)


	def connect_to_create_instance(self, entity):
		""" Create new contact records when creating a new instance of given entity.
		"""

		# scope variable is shared between callbacks
		scope = {}

		# catch the incoming data to use after entity instance is created
		def catch_incoming_data(o,data):

			scope['contactData'] = None

			if 'contacts' in data:
				scope['contactData'] = data.pop('contacts',[])


		# create contacts from given data and associate with entity instance
		def create_contacts(o,instance):

			for contact in scope['contactData']:
			    contact['progenitor'] = instance.moniker()

			serializer = ContactSerializer(data=scope['contactData'], many=True)
			if serializer.is_valid():
			    recordSet = serializer.save()

		# register callbacks
		on(entity+'.create:before',catch_incoming_data)
		on(entity+'.create:success',create_contacts) 

	def connect_to_delete_instance(entity):

    	# scope variable is shared between callbacks
		scope = {}

		def before(o,instance):
			scope['query'] = Contacts.objects.filter(progenitor=instance.moniker())

		def delete_contacts(o,instance):
			scope['query'].delete()

		on(entity+'.delete:before',before)
		on(entity+'.delete:success',delete_contacts)





from django.conf.urls import url, include
from .views import UserViewSet, AuthenticationView, ResetPasswordRequestView, ValidateEmailView

# build router
from rest_framework.routers import DefaultRouter

router = DefaultRouter()
router.register(r'users', UserViewSet, base_name='users')

urlpatterns = [
    url(r'^',include(router.urls)),
    url(r'^authentication/$', AuthenticationView.as_view(), name='authentication'),
    url(r'^authentication/password/((?P<key>[A-Z0-9_]+)/)?$', ResetPasswordRequestView.as_view(), name='reset-password'),
    url(r'^authentication/email/?$',ValidateEmailView.as_view(), name='validate-email'),
]

    # users/
    # 	- GET: list
    # 	- POST: register
    # 	- PATCH: update
    
    # authentication/
    # 	- POST: login
    # 	- DELETE: logout
    # 	- PATCH: activate account

    # authentication/password
    # 	- POST: create password reset request
    # 	- PATCH: modify password
    	
    # authentication/email
    # 	- GET: verify email
    # 		- parameters:
    # 			email
    # 			userid


# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-04 11:09
from __future__ import unicode_literals

import agape.authentication.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates whether user can log into the backend.')),
                ('status', models.SmallIntegerField(choices=[('', '-- select --'), (-1, 'Disabled'), (0, 'Inactive'), (1, 'Active')], default=0)),
                ('last_login', models.DateTimeField(blank=True, null=True)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified', models.DateTimeField(auto_now=True, null=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='created_user', to=settings.AUTH_USER_MODEL)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('modified_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='modified_user', to=settings.AUTH_USER_MODEL)),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ResetPasswordRequest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(default=agape.authentication.models.ResetPasswordRequest.generate_key, max_length=16, unique=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('status', models.SmallIntegerField(choices=[('', '-- select --'), (-1, 'Disabled'), (1, 'Active'), (2, 'Used')], default=1)),
                ('used', models.DateTimeField(blank=True, null=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='UserActivation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(default=agape.authentication.models.UserActivation.generate_key, max_length=8, unique=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('activated', models.DateTimeField(blank=True, null=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from rest_framework import permissions,status,views,viewsets
from rest_framework.response import Response
from rest_framework import generics


from .models import Contact
from .serializers import ContactSerializer

from agape.signals import on



class ContactViewSet(viewsets.ModelViewSet):
	""" Viewset that provides CRUD operations for people.

	Extends:
		viewset.ModelViewSet

	"""

	queryset = Contact.objects.all()
	serializer_class = ContactSerializer
 
def connect_to_people():

	def before(request, data, scope):
		scope = {}
		scope['concats'] = data.pop('contacts')

	def success(request, instance, scope):
		for i in trigger.contactData:
			i.progenitor = instance.moniker()

		serializer = ContactSerializer(data=breath.contactData,multiple=True)
 

	on('agape.authentication.user.create:before', before, scope)
	on('agape.authentication.user.create:success', success, scope)








from rest_framework import serializers

from .models import Person

class PersonSerializer(serializers.ModelSerializer):

	id = serializers.ReadOnlyField()

	class Meta:
		model = Person

		fields = ('id', 'title', 'first_name', 'middle_name', 'last_name', 'gender', 'birthday')

		read_only_fields = ['id']




from django.test import TestCase
from rest_framework.test import APIRequestFactory
from rest_framework.test import APIClient

import json
from django.utils import timezone
from datetime import timedelta

# Create your tests here.
from .models import Contact
from .serializers import ContactSerializer

from agape.people.models import Person
from agape.people.serializers import PersonSerializer

from agape.signals import on

# from .settings import AUTHENTICATION

class ContactTestCase(TestCase):

    def setUp(self):
        self.person = Person(first_name="Elvis",middle_name="Aaron",last_name="Presley", birthday="1935-01-08", gender="m")
        self.person.save()
        
    def test_sanity(self):
        self.assertTrue(True, "Sane")

    def test_create(self):
        instance = Contact(progenitor=self.person.moniker(), type=Contact.TELEPHONE, label="Home", value="+507 5555 5555")
        instance.save()

        self.assertEqual(instance.id, 1, 'Created contact')
        self.assertEqual(instance.progenitor, 'person:1', 'Progenitor set')
        self.assertEqual(instance.type, Contact.TELEPHONE, 'Contact type set')
        self.assertEqual(instance.value, '+507 5555 5555', 'Contact value set')


class ContactSerializerTestCase(TestCase):

    def setUp(self):
        self.person = Person(first_name="Elvis",middle_name="Aaron",last_name="Presley", birthday="1935-01-08", gender="m")
        self.person.save()

    def test_serialize(self):
        instance = Contact(progenitor=self.person.moniker(), type=Contact.TELEPHONE, label="Home", value="+507 5555 5555")
        instance.save()

        serializer = ContactSerializer(instance)

        expect = {
            'id':1,
            'progenitor':'person:1',
            'type':Contact.TELEPHONE,
            'value':'+507 5555 5555',
            'label':'Home'
        }

        self.assertDictEqual(expect, serializer.data)

    def test_inflate(self):

        data = {
            'progenitor':'person:1',
            'type':Contact.TELEPHONE,
            'value':'+507 5555 5555',
            'label':'Home'
        }

        # serializer data is valid
        serializer = ContactSerializer(data=data)
        self.assertTrue(serializer.is_valid(), 'Serializer data is valid')

        # create an instance via the serializer
        instance = serializer.create(serializer.validated_data)
        self.assertTrue(instance, 'Created instance from serializer data')

        # verify instance values
        self.assertEqual(instance.id, 1, 'Contact assigned new ID')
        self.assertEqual(instance.type, Contact.TELEPHONE)
        self.assertEqual(instance.progenitor, 'person:1')
        self.assertEqual(instance.value, '+507 5555 5555')
        self.assertEqual(instance.label, "Home")


        # verify partial modification
        data = {
            'id': instance.id,
            'value': '+507 4444 4444'
        }
        serializer = ContactSerializer(data=data,partial=True)
        self.assertTrue(serializer.is_valid(), 'Serializer data is valid')

        # update the instance
        serializer.update(instance,data)
        self.assertEqual(instance.value, '+507 4444 4444')

class APITestCase(TestCase):

    def setUp(self):
        self.client =  APIClient()
        self.person = Person(first_name="Elvis",middle_name="Aaron",last_name="Presley", birthday="1935-01-08", gender="m")
        self.person.save()
 

    def test_create_contact(self):
        data = {
            'progenitor':'person:1',
            'type':Contact.TELEPHONE,
            'value':'+507 5555 5555',
            'label':'Home'
        }      
        response = self.client.post('/api/v1/contacts/', data)
        self.assertEqual(response.status_code, 201, "Created new contact")
        
        self.assertEqual(response.data.get('id'), 1, 'Contact assigned new ID')
        self.assertEqual(response.data.get('progenitor'), "person:1")
        self.assertEqual(response.data.get('type'), Contact.TELEPHONE )
        self.assertEqual(response.data.get('value'), '+507 5555 5555')
        self.assertEqual(response.data.get('label'), "Home" )

        # verify actual database record was created
        instance = Contact.objects.get(id=response.data.get('id'))
        self.assertTrue(instance)

    def test_retrieve(self):

        data = {
            'progenitor':'person:1',
            'type':Contact.TELEPHONE,
            'value':'+507 5555 5555',
            'label':'Home'
        }      
        response = self.client.post('/api/v1/contacts/', data)
        self.assertEqual(response.status_code, 201, "Created new contact")
        
        response = None
        response = self.client.get('/api/v1/contacts/1/')
        self.assertEqual(response.data.get('id'), 1, 'Contact assigned new ID')
        self.assertEqual(response.data.get('progenitor'), "person:1")
        self.assertEqual(response.data.get('type'), Contact.TELEPHONE )
        self.assertEqual(response.data.get('value'), '+507 5555 5555')
        self.assertEqual(response.data.get('label'), "Home" )

        
    def test_update_contact(self):

        data = {
            'progenitor':'person:1',
            'type':Contact.TELEPHONE,
            'value':'+507 5555 5555',
            'label':'Home'
        }      
        response = self.client.post('/api/v1/contacts/', data)
        self.assertEqual(response.status_code, 201, "Created new contact")


        data = { 'value':'+507 4444 4444' } 
        response = self.client.patch( 
            '/api/v1/contacts/{}/'.format(response.data.get('id')),
             json.dumps(data),
              content_type='application/json')
        self.assertEqual(response.data.get('value'), '+507 4444 4444')



    def test_delete_contact(self):
        data = {
            'progenitor':'person:1',
            'type':Contact.TELEPHONE,
            'value':'+507 5555 5555',
            'label':'Home'
        }      
        response = self.client.post('/api/v1/contacts/', data)
        self.assertEqual(response.status_code, 201, "Created new contact")
        id = response.data.get('id')

        uri = '/api/v1/contacts/{}/'.format(response.data.get('id'))
        response = self.client.delete( uri )    
        self.assertEqual(response.status_code, 204, "Deleted")

        query = Contact.objects.filter(id=id)
        self.assertEqual(len(query),0, "Deleted")


class ContactsConnectionTestCase(TestCase):

    def setUp(self):
        self.client =  APIClient()

        from .connector import ContactsConnector

        connector = ContactsConnector()
        connector.connect_to_entity('person')
        # connector.connect('agape.people')


    def test_create(self):

        data = {
            'first_name':'Elvis',
            'middle_name':'Aaron',
            'last_name':'Presley',
            'birthday': "1935-01-08",
            'gender': 'm',
            'contacts': [
                {'type':Contact.TELEPHONE, 'label':'Home', 'value':'Priceless'},
                {'type':Contact.EMAIL, 'label':'Work', 'value':'test@example.com'},
            ]
        }        
        response = self.client.post('/api/v1/people/', data)
        self.assertEqual(response.status_code, 201, "Created new person")

        # find the contact that was created
        contact = Contact.objects.get(id=1)
        self.assertEqual(contact.value, 'Priceless', "Created phone record")

        contact = Contact.objects.get(id=2)
        self.assertEqual(contact.value, 'test@example.com', "Created email record")




# class PeopleContactsConnectionTestCase(TestCase):

#     def setUp(self):
#         self.client =  APIClient()

#         # create callbacks on person creation
#         scope = {}

#         def catch_incoming_data(o,request):
#             scope['contactData'] = None
#             scope['contactData'] = request.data.get('contacts',[])

#         def create_contacts(o,person):

#             print( scope['contactData'] );

#             for contact in scope['contactData']:
#                 contact['progenitor'] = person.moniker()

#             serializer = ContactSerializer(data=scope['contactData'], many=True)
#             if serializer.is_valid():
#                 recordSet = serializer.save()

#         on('person.create:before',catch_incoming_data)
#         on('person.create:success',create_contacts)

#     def test_create_contacts(self):

#         data = {
#             'first_name':'Elvis',
#             'middle_name':'Aaron',
#             'last_name':'Presley',
#             'birthday': "1935-01-08",
#             'gender': 'm',
#             'contacts': [
#                 {'type':Contact.TELEPHONE, 'label':'Home', 'value':'Priceless'},
#                 {'type':Contact.EMAIL, 'label':'Work', 'value':'test@example.com'},
#             ]
#         }        
#         response = self.client.post('/api/v1/people/', data)
#         self.assertEqual(response.status_code, 201, "Created new person")

#         # find the contact that was created
#         contact = Contact.objects.get(id=1)
#         self.assertEqual(contact.value, 'Priceless', "Created phone record")

#         contact = Contact.objects.get(id=2)
#         self.assertEqual(contact.value, 'test@example.com', "Created email record")




from django.conf.urls import url, include
from .views import OrganizationViewSet

# build router
from rest_framework.routers import DefaultRouter

router = DefaultRouter()
router.register(r'organizations', OrganizationViewSet, base_name='organizations')

urlpatterns = [
    url(r'^',include(router.urls)),
]


from agape import viewsets

from .models import Organization
from .serializers import OrganizationSerializer


class OrganizationViewSet(viewsets.ModelViewSet):
	""" Viewset that provides CRUD operations for organization.

	Extends:
		viewset.ModelViewSet

	"""

	queryset = Organization.objects.all()
	serializer_class = OrganizationSerializer
	context = 'organization'






# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-04 11:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('progenitor', models.CharField(blank=True, max_length=64, null=True)),
                ('type', models.CharField(blank=True, choices=[('', '-- select --'), ('Telephone', 'Telephone'), ('Email', 'Email'), ('Chat', 'Chat'), ('Profile', 'Profile'), ('Website', 'Website')], max_length=16, null=True)),
                ('value', models.CharField(blank=True, max_length=255, null=True)),
                ('label', models.CharField(blank=True, max_length=32, null=True)),
            ],
        ),
    ]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='agape-django',
    version='0.3.1',
    packages=['agape','agape.authentication','agape.contacts','agape.organizations','agape.people','agape.members','agape.files'],
    include_package_data=True,
    license='MIT License', 
    description='Collection of reusable Django Applications for Rapid Application Development',
    long_description=README,
    url = 'https://github.com/codewiseio/agape-django',
    author='Jeffrey Ray Hallock',
    author_email='codewiseio@gmail.com',
    test_suite = "runtests.runtests",
    install_requires = [
        'Django>=1.11.5',
        'django-cors-headers>=2.1.0',
        'django-filter>=1.1.0',
        'djangorestframework>=3.6.4',
        'djangorestframework-jwt>=1.11.0'
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        # Replace these appropriately if you are stuck on Python 2.
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)




observers = {}

class Observer():

	context  = None
	method   = None
	data     = None

	def __init__(self, context, method, data=None):
		self.context = context
		self.method  = method
		self.data = data

	def destroy(self):
		observers[self.context].remove(self)

	def react(self,*args):
		self.method(self,*args)


def on(context, method, data=None):

	observer = Observer(context,method,data)
	observers[context] = observers.get(context,[])
	observers[context].append( observer )
	return observer

def trigger(context, *args,**kwargs):

	for observer in observers.get(context,[]):
		observer.react(*args)




# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-12-04 11:08
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('progenitor', models.CharField(blank=True, max_length=64, null=True)),
                ('name', models.CharField(max_length=255)),
                ('slug', models.SlugField(allow_unicode=True, blank=True, max_length=255)),
            ],
        ),
    ]



from django.test import TestCase
from rest_framework.test import APIRequestFactory
from rest_framework.test import APIClient
import json
from django.utils import timezone
from datetime import timedelta

# Create your tests here.
from .models import User, UserManager, UserActivation, ResetPasswordRequest
from .settings import AUTHENTICATION

class UserTestCase(TestCase):
    def setUp(self):
       user = User(email="codewiseio@gmail.com")
       user.set_password("password")
       user.save()


    def test_user_account(self):
        """User account is created and retrieved."""
        user = User.objects.get(email="codewiseio@gmail.com")
        self.assertEqual(user.email, 'codewiseio@gmail.com')

    def test_account_activation(self):
    	user = User.objects.get(email="codewiseio@gmail.com")
    	instance = UserActivation.objects.create(user=user)
    	self.assertTrue(instance.key, "Created activation key")

    def test_password_reset(self):
      user = User.objects.get(email="codewiseio@gmail.com")
      instance = ResetPasswordRequest.objects.create(user=user)
      self.assertTrue(instance.key, "Created activation key")   

class UserManagerTestCase(TestCase):
    
    def test_create_user(self):
        user_manager = UserManager()
        user = user_manager.create_user('test@example.com','testingpassword')

        user = User.objects.get(email='test@example.com')
        self.assertEqual(user.email,  'test@example.com')

    def test_create_superuser(self):
        user_manager = UserManager()
        user = user_manager.create_superuser('admin@example.com','testingpassword')  

        user = User.objects.get(email="admin@example.com")
        self.assertEqual(user.email,  'admin@example.com')     





class APITestCase(TestCase):

    def setUp(self):
        self.client =  APIClient()


    def test_user_registration(self):
        # register user
        response = self.client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 201, "Created new user")
        
        # get user, verify that user is not active
        user = User.objects.get(email="testing@example.com")
        self.assertEqual(user.status, 0, "New user is inactive")

        # attempt to login with credentials (should fail because user is inactive)
        response = self.client.post('/api/v1/authentication/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 401, "Could not login to inactive account")

        # check if activation link created
        activation =  UserActivation.objects.get(user_id=user.id)
        self.assertTrue(activation, "User activation key generated")
        self.assertTrue(not activation.activated, "Activation has not been used");

        # follow activation link
        response = self.client.patch('/api/v1/authentication/', json.dumps({'key':activation.key}), content_type='application/json')
        self.assertEqual(response.status_code, 200, "User has been activated")

        # check if activation link is used
        activation =  UserActivation.objects.get(key=activation.key)
        self.assertTrue(activation.activated, "Activation key has been used");

        # check if user is now active
        user = User.objects.get(email="testing@example.com")
        self.assertEqual(user.status, 1, "User is now active")

        # attempt to login with credentials (should pass because account has been activated)
        response = self.client.post('/api/v1/authentication/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 200, "Sign in successful")
        self.assertEqual(response.data['user'].get('email'), "testing@example.com")
        self.assertTrue(response.data.get('token'), "Received authentication token.")

        # logout
        response = self.client.delete('/api/v1/authentication/')
        self.assertEqual(response.status_code, 204, "Sign out successful")

    def test_register_without_user_activation(self):
        AUTHENTICATION['REQUIRE_ACTIVATION'] = False

        # register user
        response = self.client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 201, "Created new user")

        # get user, verify that user is not active
        user = User.objects.get(email="testing@example.com")
        self.assertEqual(user.status, 1, "New user active")

        AUTHENTICATION['REQUIRE_ACTIVATION'] = True


    def test_authenticate_disabled_account_error(self):
        # register user
        response = self.client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 201, "Created new user")

        # disable user
        user = User.objects.get(email="testing@example.com")
        user.status = -1;
        user.save();

        user = User.objects.get(email="testing@example.com")
        self.assertEqual(user.status, -1, "New user is disabled")

        # check if activation link created
        activation =  UserActivation.objects.get(user_id=user.id)
        self.assertTrue(activation, "User activation key generated")
        self.assertTrue(not activation.activated, "Activation has not been used");

        # follow activation link (expect an error)
        response = self.client.patch('/api/v1/authentication/', json.dumps({'key':activation.key}), content_type='application/json')
        self.assertEqual(response.status_code, 400, "Can not activate disabled user")

    def test_register_duplicate_email_error(self):
        # register user
        response = self.client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 201, "Created new user")

        response = self.client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 409, "Could not register duplicate account")

    def test_password_reset(self):
        # register user
        response = self.client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 201, "Created new user")    

        # activate user
        user = User.objects.get(email="testing@example.com")
        user.status = 1;
        user.save()

        # create password reset request
        response = self.client.post('/api/v1/authentication/password/', {'email': 'testing@example.com', 'reset': True})
        self.assertEqual(response.status_code, 200, "Reset password request created")

        # check for existence of database entry
        instance = ResetPasswordRequest.objects.filter(user=user).order_by('-id')[0]
        self.assertTrue(instance, "Reset password request object exists")

        # check password reset link is valid
        response = self.client.get('/api/v1/authentication/password/{}/'.format(instance.key))
        self.assertEqual(response.status_code, 200, "Validate password reset link ok")   

        # try creating another password reset and verify that they have the same key
        response = self.client.post('/api/v1/authentication/password/', {'email': 'testing@example.com', 'reset': True})
        duplicate = ResetPasswordRequest.objects.filter(user=user).order_by('-id')[0]
        self.assertEqual(instance.key, duplicate.key, "Used the same password reset request")   

        # update the original reset request to be over an hour old
        duplicate = None
        time_threshold = timezone.now() - timedelta(minutes=AUTHENTICATION.get('RESET_PASSWORD_EXPIRES')+1)
        instance.created = time_threshold
        instance.save()

        # check password reset link is now expired
        response = self.client.get('/api/v1/authentication/password/{}/'.format(instance.key))
        self.assertEqual(response.status_code, 410, "Password link has expired")

        # reset the password using the expired link (should fail)
        response = self.client.patch('/api/v1/authentication/password/{}/'.format(instance.key), json.dumps({'password': 'newpass'}), content_type='application/json')
        self.assertEqual(response.status_code, 410, "Do not reset password on expired link")


        # create another password reset and verify that they have different keys
        response = self.client.post('/api/v1/authentication/password/', {'email': 'testing@example.com', 'reset': True})
        duplicate = ResetPasswordRequest.objects.filter(user=user).order_by('-id')[0]
        self.assertTrue(not instance.key == duplicate.key, "Created new password reset request") 

        # reset the password using the new link (should pass)
        response = self.client.patch('/api/v1/authentication/password/{}/'.format(duplicate.key), json.dumps({'password': 'newpass'}), content_type='application/json')
        self.assertEqual(response.status_code, 200, "Reset password on valid link")

        # check password reset link is now expired (has been used)
        response = self.client.get('/api/v1/authentication/password/{}/'.format(instance.key))
        self.assertEqual(response.status_code, 410, "Password link has been used")

        # reset the password using the new link again (should fail)  
        response = self.client.patch('/api/v1/authentication/password/{}/'.format(duplicate.key), json.dumps({'password': 'newpass'}), content_type='application/json')
        self.assertEqual(response.status_code, 410, "Do not reset on used link")

        # create a new password reset request, validate that it is a unique key
        used = duplicate

        # create another password reset and verify that they have different keys
        response = self.client.post('/api/v1/authentication/password/', {'email': 'testing@example.com', 'reset': True})
        another = ResetPasswordRequest.objects.filter(user=user).order_by('-id')[0]
        self.assertTrue(not used.key == another.key, "Created new password reset request") 

        # submit the change request with no password (should fail)
        response = self.client.patch('/api/v1/authentication/password/{}/'.format(another.key), json.dumps({'password': ''}), content_type='application/json')
        self.assertEqual(response.status_code, 400, "No password supplied error")

        # set the new link to disabled
        another.status = ResetPasswordRequest.DISABLED
        another.save()

        # check password reset link is now disabled
        response = self.client.get('/api/v1/authentication/password/{}/'.format(another.key))
        self.assertEqual(response.status_code, 403, "Link has been disabled")

        # try resetting password on disbaled link
        response = self.client.patch('/api/v1/authentication/password/{}/'.format(another.key), json.dumps({'password': 'newpass'}), content_type='application/json')
        self.assertEqual(response.status_code, 403, "Link has been disabled")

        # set the new link to enabled
        another.status = ResetPasswordRequest.ENABLED
        another.save()

        # set the user to disabled
        another.user.status = User.DISABLED
        another.user.save()

        # check password reset link is now disabled
        response = self.client.get('/api/v1/authentication/password/{}/'.format(another.key))
        self.assertEqual(response.status_code, 403, "User has been disabled")

        # try resetting password on disbaled link
        response = self.client.patch('/api/v1/authentication/password/{}/'.format(another.key), json.dumps({'password': 'newpass'}), content_type='application/json')
        self.assertEqual(response.status_code, 403, "User has been disabled")    


    def test_validate_email(self):

        # check if email is available ( yes available )
        response = self.client.get('/api/v1/authentication/email/?email=testing@example.com')
        self.assertEqual(response.data.get('valid'), True, "No conflict")

        # register user
        response = self.client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 201, "Created new user")    


        user = User.objects.get(email="testing@example.com")

        # check if email is available ( no, in use )
        response = self.client.get('/api/v1/authentication/email/?email=testing@example.com')
        self.assertEqual(response.data.get('valid'), False, "Conflict")

        # check if email is available ( yes, do not flag users email as invalid )
        response = self.client.get('/api/v1/authentication/email/?email=testing@example.com&userid={}'.format(user.id))
        self.assertEqual(response.data.get('valid'), True, "No conflict")

    def test_change_credentials(self):

        # create a user
        client =  APIClient()
        response = client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 201, "Created new user")

        user = User.objects.get(email="testing@example.com")
        user.status = 1;
        user.save()

        # login
        response = client.post('/api/v1/authentication/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 200, "Signed in")
        client.credentials(HTTP_AUTHORIZATION='Bearer ' + response.data.get('token'))

        # update password
        userid = response.data['user'].get('id')    
        response = client.patch('/api/v1/users/{}/'.format(userid), json.dumps({'password':'newpass','confirm_password':'testing'}), content_type='application/json')
        self.assertEqual(response.status_code, 200, "Updated password")    

        # update email 
        response = client.patch('/api/v1/users/{}/'.format(userid), json.dumps({'email':'updated@example.com','confirm_password':'newpass'}), content_type='application/json')
        self.assertEqual(response.status_code, 200, "Updated password")   
        

        # login with new credentials
        response = client.post('/api/v1/authentication/', {'email': 'updated@example.com', 'password':'newpass'})
        self.assertEqual(response.status_code, 200, "Signed in")
        client.credentials(HTTP_AUTHORIZATION='Bearer ' + response.data.get('token'))



    def test_last_login(self):

        AUTHENTICATION['REQUIRE_ACTIVATION'] = False

        # create a user
        client =  APIClient()
        response = client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 201, "Created new user")

        # verify last login is blank
        user = User.objects.get(email="testing@example.com")
        self.assertEqual(user.last_login, None, "User has never logged in")

        # login
        response = client.post('/api/v1/authentication/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 200, "Signed in")

        # verify last login is not blank
        user = User.objects.get(email="testing@example.com")
        self.assertTrue(user.last_login, "User has logged in")

        AUTHENTICATION['REQUIRE_ACTIVATION'] = True

  
    def test_list(self):

        # register user
        response = self.client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 201, "Created new user")

        # activate user
        user = User.objects.get(email="testing@example.com")
        user.status = 1;
        user.save()

        # add list of users
        response = self.client.post('/api/v1/users/', {'email': 'user1@example.com', 'password':'testing'})
        response = self.client.post('/api/v1/users/', {'email': 'user2@example.com', 'password':'testing'})
        response = self.client.post('/api/v1/users/', {'email': 'user3@example.com', 'password':'testing'})
        response = self.client.post('/api/v1/users/', {'email': 'user4@example.com', 'password':'testing'})
        response = self.client.post('/api/v1/users/', {'email': 'user5@example.com', 'password':'testing'})
        response = self.client.post('/api/v1/users/', {'email': 'user6@example.com', 'password':'testing'})
        
        # list user (without permissions)
        response = self.client.get('/api/v1/users/')
        self.assertEqual(response.status_code, 403, "List users unauthorized")  

        # add permissions to list users
        user.is_superuser = True
        user.save()   

        # login
        response = self.client.post('/api/v1/authentication/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 200, "Signed in")
        self.client.credentials(HTTP_AUTHORIZATION='Bearer ' + response.data.get('token'))

        # list user (with super admin permissions)
        response = self.client.get('/api/v1/users/')
        self.assertEqual(response.status_code, 200, "List users as super admin")  




    def test_permissions(self):

        # register user
        response = self.client.post('/api/v1/users/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 201, "Created new user")

        # activate user
        user = User.objects.get(email="testing@example.com")
        user.status = 1;
        user.save()

        # retrieve user (unauthorized)
        userid = response.data['id']
        response = self.client.get('/api/v1/users/{}/'.format(userid))
        self.assertEqual(response.status_code, 403, "Retrieve user unauthorized")     

        # login
        response = self.client.post('/api/v1/authentication/', {'email': 'testing@example.com', 'password':'testing'})
        self.assertEqual(response.status_code, 200, "Signed in")
        self.client.credentials(HTTP_AUTHORIZATION='Bearer ' + response.data.get('token'))
        
        # retrieve user (authorized)
        userid = response.data['user']['id']
        response = self.client.get('/api/v1/users/{}/'.format(userid))
        self.assertEqual(response.status_code, 200, "Retrieved user data")    



                



from django.test import TestCase
from rest_framework.test import APIRequestFactory
from rest_framework.test import APIClient

import json
from django.core.files.uploadedfile import SimpleUploadedFile
from django.conf import settings
from django.utils import timezone
from datetime import timedelta

import os.path
import shutil

# Create your tests here.
from .models import File
from .serializers import FileSerializer

from agape.people.models import Person
from agape.people.serializers import PersonSerializer

from agape.signals import on


MODEL = File
SERIALIZER_CLASS = FileSerializer
CREATE_DATA =  {   
        'name': 'blue-man-file.pdf',
        'type': 'pdf',
        'path': 'aso0w98we098ds09.pdf',

        'label': 'Blue Man File',
        'description': 'Guide to creating the perfect person.',

    }
EXPECT_DATA = {   
    'id':1,
    'progenitor': None,
    'name': 'blue-man-file.pdf',
    'type': 'pdf',
    'path': 'aso0w98we098ds09.pdf',

    'label': 'Blue Man File',
    'description': 'Guide to creating the perfect person.',
    }

UPDATE_DATA = { 
    'description': 'I am that I am.'
}
UPDATE_EXPECT_DATA= {   
    'id':1,
    'label':'Blue Man File',
    'description': 'I am that I am.'
}

class ModelTestCase(TestCase):

    def setUp(self):
        self.model              = MODEL
        self.serializer_class   = SERIALIZER_CLASS
        self.create_data        = CREATE_DATA
        self.expect_data        = EXPECT_DATA
        self.update_data        = UPDATE_DATA
        self.update_expect_data = UPDATE_EXPECT_DATA

    def test_create(self):
        instance = self.model(**self.create_data)
        instance.save()

        # compare instance to expected data
        for key in self.expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_update(self):
        instance = self.model(**self.create_data)
        instance.save()

        # update the instance
        for key, value in self.update_data.items():
            instance_value = setattr(instance, key, value )
        
        # save the instance
        instance.save()

        # retrieve the instance
        instance_id = instance.id
        instance = None
        instance = self.model.objects.get(id=instance_id)

        # compare instance to expected data
        for key in self.update_expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.update_expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )



class SerializerTestCase(TestCase):

    def setUp(self):
        self.model              = MODEL
        self.serializer_class   = SERIALIZER_CLASS
        self.create_data        = CREATE_DATA
        self.expect_data        = EXPECT_DATA
        self.update_data        = UPDATE_DATA
        self.update_expect_data = UPDATE_EXPECT_DATA

    def test_serialize(self):
        instance =self.model(**self.create_data)
        instance.save()

        serializer = self.serializer_class(instance)

        # compare instance to expected data
        for key in self.expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )


    def test_create(self):

        # serializer data is valid
        serializer = self.serializer_class(data=self.create_data)
        self.assertTrue(serializer.is_valid(), 'Serializer data is valid')

        # create an instance via the serializer
        instance = serializer.create(serializer.validated_data)
        self.assertTrue(instance, 'Created instance from serializer data')

        # compare instance to expected data
        for key in self.expect_data.keys():
            instance_value = getattr(instance, key)
            expected_value = self.expect_data[key]
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_update(self):
        instance =self.model(**self.create_data)
        instance.save()

        serializer = self.serializer_class(data=self.update_data)
        self.assertTrue(serializer.is_valid(), 'Serializer data is valid')

        instance = serializer.update(instance, serializer.validated_data)

        # compare instance to expected data
        for key, expected_value in self.update_expect_data.items():
            instance_value = getattr(instance, key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

class APITestCase(TestCase):

    def setUp(self):
        self.model              = MODEL
        self.serializer_class   = SERIALIZER_CLASS

        self.create_data        = CREATE_DATA.copy()
        del self.create_data['path']
        self.create_data['file'] =  SimpleUploadedFile("file.mp4", b"file_content", content_type="video/mp4")

        self.expect_data        = EXPECT_DATA.copy()
        del self.expect_data['path']
        self.update_data        = UPDATE_DATA.copy()

        self.update_expect_data = UPDATE_EXPECT_DATA

        self.client =  APIClient()
        self.entity = 'group'
        self.api_end_point = '/api/v1/files/'

    def test_no_file_supplied(self):
        response = self.client.post(self.api_end_point, self.create_data)
        self.assertEqual(response.status_code, 400, "Created new instance")


    def test_create(self):
        response = self.client.post(self.api_end_point, self.create_data, format='multipart')
        self.assertEqual(response.status_code, 201, "Created new instance")

        # compare response to expected data
        for key, expected_value in self.expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

        # verify actual database record was created
        instance = self.model.objects.get(id=response.data.get('id'))
        self.assertTrue(instance)

        # verify that the uploaded file exists on disk
        self.assertTrue(os.path.isfile(instance.full_path()));

    def test_retrieve(self):
        response = self.client.post(self.api_end_point, self.create_data, format='multipart')
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)
        self.assertEqual(response.status_code, 201, "Retrieved")

        response = None
        response = self.client.get(uri)

        # compare response to expected data
        for key, expected_value in self.expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

    def test_download(self):
        response = self.client.post(self.api_end_point, self.create_data, format='multipart')
        self.assertEqual(response.status_code, 201, "Created new instance")

        # TODO: Test file download

        instance_id = response.data.get('id')
        uri = "{}{}/".format(self.api_end_point,instance_id)
        print(uri)

        response = self.client.get(uri)

        self.assertEqual(response.status_code, 200, "Created new instance")


    def test_update(self):
        response = self.client.post(self.api_end_point, self.create_data, format='multipart')
        self.assertEqual(response.status_code, 201, "Created new instance")

        instance_id = response.data.get('id')
        original_instance = File.objects.get(id=instance_id)
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = self.client.patch(uri, json.dumps(self.update_data), content_type='application/json')
        self.assertEqual(response.status_code, 200, "Updated instance")

        # validate response
        for key, expected_value in self.update_expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

        # perform get operation and validate
        instance_id = response.data.get('id')
        instance = File.objects.get(id=instance_id)
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = None
        response = self.client.get(uri)

        # compare response to expected data
        for key, expected_value in self.update_expect_data.items():
            instance_value = response.data.get(key)
            self.assertEqual(instance_value, expected_value, "Comparing {} attribute".format(key) )

        # self.assertTrue(original_instance.path != instance.path )

    def test_delete(self):
        response = self.client.post(self.api_end_point, self.create_data, format='multipart')
        self.assertEqual(response.status_code, 201, "Created new instance")


        instance_id = response.data.get('id')
        instance = File.objects.get(id=instance_id)
        uri = "{}{}/".format(self.api_end_point,instance_id)

        response = self.client.delete( uri )    
        self.assertEqual(response.status_code, 204, "Deleted")

        query = self.model.objects.filter(id=instance_id)
        self.assertEqual(len(query),0, "Deleted")

        # verify that the uploaded file exists on disk
        self.assertTrue(not os.path.isfile(instance.full_path()));

    def tearDown(self):
        if (os.path.isdir(settings.MEDIA_ROOT)):
            shutil.rmtree(settings.MEDIA_ROOT)
